/* AppControler.m */

/*
 Copyright (c) 2005 University of Utah Student Computing Labs.
 All Rights Reserved.

 Permission to use, copy, modify, and distribute this software and
 its documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appears in all copies and
 that both that copyright notice and this permission notice appear
 in supporting documentation, and that the name of The University
 of Utah not be used in advertising or publicity pertaining to
 distribution of the software without specific, written prior
 permission. This software is supplied as is without expressed or
 implied warranties of any kind.
*/

#import "AppController.h"
#import <fcntl.h>
#import <sys/stat.h>
#import <unistd.h>

// Stuff used in the pref file

#define PREF_TITLE_NAME			@"title"
#define PREF_IMAGE_NAME			@"status"
#define PREF_IMAGE_NORMAL		@"normal"
#define PREF_IMAGE_ERROR		@"error"
#define PREF_MENU_IMAGE_PATH	@"menuImagePath"
#define PREF_MENU_ARRAY			@"menuArray"
#define PREF_MENU_ITEM_TEXT		@"menuTitle"
#define PREF_MENU_ACTION		@"menuAction"
#define PREF_MENU_INDENT		@"menuIndentationLevel"
#define MENU_SEPARATOR			@"separator"
#define ACTION_LOGOUT			@"logout"
#define ACTION_RESET			@"reset"
#define ACTION_EDIT_PREFS		@"editMyPrefs"
#define ACTION_QUIT				@"quit"

/*
<dict>
	<key>PREF_TITLE_NAME</key>
	<string></string>
	<key>PREF_IMAGE_NAME</key>
	<string></string>
	<key>PREF_MENU_IMAGE_PATH</key>
	<string></string>
	<key>PREF_MENU_ARRAY</key>
	<array>
		<dict>
			<key>PREF_MENU_ITEM_TEXT</key>
			<string></string>
			<key>PREF_MENU_ACTION</key>
			<string></string>
			<key>PREF_MENU_INDENT</key>
			<string></string>
		</dict>
		<dict>
			<key>PREF_MENU_ITEM_TEXT</key>
			<string></string>
			<key>PREF_MENU_ACTION</key>
			<string></string>
			<key>PREF_MENU_INDENT</key>
			<string></string>
		</dict>
	</array>
</dict>
*/

// Other stuff

#define STATUS_BAR_IMAGE_NORMAL	@"status_green.tif"
#define STATUS_BAR_IMAGE_ERROR	@"status_red.tif"
#define LOGOUT_IMAGE_NAME		@"target_18_18.tif"

#define LOGOUT_TRIGGER_FILE		"/tmp/logoutNow"
#define RESET_TRIGGER_FILE		"/tmp/resetNow"

@implementation AppController

// LOGOUT VARIABLES
int timeout = 30;
int counter;
BOOL loggingout = NO;
BOOL resetting = NO;
BOOL loginWindowShowing = NO;
BOOL resetWindowShowing = NO;

- (void)awakeFromNib
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString *menu_image_path;
	NSString *default_image;
	NSString *title;
	NSArray *menuItems;

	// Create status item
	statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength] retain];
	
	// Populate the menu with stuff
	if ([[[NSProcessInfo processInfo] processName] isEqualToString:@"LogoutMenu"]) {

		// Don't bother setting/reading prefs, just set the values.

		title = @"Logout";
		menu_image_path = @"";
		default_image = ACTION_LOGOUT;
		menuItems = [NSArray arrayWithObjects:
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Log me out",ACTION_LOGOUT,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			nil];

	} else if ([[[NSProcessInfo processInfo] processName] isEqualToString:@"ResetKiosk"]) {

		// Don't bother setting/reading prefs, just set the values.

		title = @"Reset";
		menu_image_path = @"";
		default_image = ACTION_RESET;
		menuItems = [NSArray arrayWithObjects:
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Reset",ACTION_RESET,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			nil];

	} else {

		// Register Defaults
		[defaultValues setObject:@"MH2.30" forKey:PREF_TITLE_NAME];
		[defaultValues setObject:@"" forKey:PREF_MENU_IMAGE_PATH];
		[defaultValues setObject:PREF_IMAGE_NORMAL forKey:PREF_IMAGE_NAME];
		// This list goes from bottom to top... kinda strange.
		NSArray *prefArray = [NSArray arrayWithObjects:
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Reset",ACTION_RESET,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Log me out",ACTION_LOGOUT,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"You should edit my prefs!",ACTION_EDIT_PREFS,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"",MENU_SEPARATOR,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Error",@"",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"I thought I saw a puddy cat, really, I thought I did.  Don't you believe me?",@"",@"1",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,PREF_MENU_INDENT,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Wait, actually, I am a cat.  Yup, that's the error, I don't know what I am.",@"",@"1",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,PREF_MENU_INDENT,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"",MENU_SEPARATOR,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Next Run",@"",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Mon, Feb. 30th, 2004 @ 2:00 AM",@"",@"1",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,PREF_MENU_INDENT,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Last Run",@"",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Mon, Feb. 30th, 2004 @ 2:00 AM",@"",@"1",nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,PREF_MENU_INDENT,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"",MENU_SEPARATOR,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			[NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"Quit",ACTION_QUIT,nil]
																  forKeys:[NSArray arrayWithObjects:PREF_MENU_ITEM_TEXT,PREF_MENU_ACTION,nil]],
			nil];
		[defaultValues setObject:prefArray forKey:PREF_MENU_ARRAY];
		[[NSUserDefaults standardUserDefaults] registerDefaults: defaultValues];

		// Read the prefs now.
		title = [defaults stringForKey:PREF_TITLE_NAME];
		menu_image_path = [[defaults stringForKey:PREF_MENU_IMAGE_PATH] stringByStandardizingPath];
		default_image = [defaults stringForKey:PREF_IMAGE_NAME];
		menuItems = [defaults arrayForKey:PREF_MENU_ARRAY];
		
	}
	
	// Setup menu.
	
	// Set title
	[statusItem setTitle:title]; 

	// Menu image stuff
	// .. from built-in images
	if ([default_image length]) {
		if ([default_image isEqualToString:PREF_IMAGE_NORMAL]) {
			[statusItem setImage:[NSImage imageNamed:STATUS_BAR_IMAGE_NORMAL]];
		} else if ([default_image isEqualToString:PREF_IMAGE_ERROR]) {
			[statusItem setImage:[NSImage imageNamed:STATUS_BAR_IMAGE_ERROR]];
		} else if ([default_image isEqualToString:ACTION_LOGOUT]) {
			[statusItem setImage:[NSImage imageNamed:LOGOUT_IMAGE_NAME]];
		} else if ([default_image isEqualToString:ACTION_RESET]) {
			[statusItem setImage:[NSImage imageNamed:LOGOUT_IMAGE_NAME]];
		}
	}
	// ... or from a path
	if([menu_image_path length]) {
		if ([fileManager fileExistsAtPath:menu_image_path]) {
			[statusItem setImage:[[[NSImage alloc] initWithContentsOfFile:menu_image_path] autorelease]];
		} else {
			NSLog(@"File not found: %@", menu_image_path);
		}
	}
	
	// Menu items
	NSEnumerator *menuEnumerator = [menuItems reverseObjectEnumerator];
	NSDictionary *menuContents;
	while (menuContents = [menuEnumerator nextObject]) {
		NSString *menu_item_text = [menuContents objectForKey:PREF_MENU_ITEM_TEXT];
		NSString *menu_item_job = [menuContents objectForKey:PREF_MENU_ACTION];
		NSString *menu_item_indentation = [menuContents objectForKey:PREF_MENU_INDENT];
		if (menu_item_text != nil && menu_item_job != nil) {

			NSMenuItem *menu_item;
			[statusItem setHighlightMode:YES];
			[statusItem setMenu:theMenu];

			if ([menu_item_job length]) {
				if ([menu_item_job isEqualToString:ACTION_LOGOUT]) {
					menu_item = [[NSMenuItem alloc] initWithTitle:menu_item_text action:@selector(openLogoutWindow:) keyEquivalent:@""];
					//[menu_item setImage:[NSImage imageNamed:LOGOUT_IMAGE_NAME]];
				} else if ([menu_item_job isEqualToString:ACTION_RESET]) {
					menu_item = [[NSMenuItem alloc] initWithTitle:menu_item_text action:@selector(openResetWindow:) keyEquivalent:@""];
					//[menu_item setImage:[NSImage imageNamed:LOGOUT_IMAGE_NAME]];
				} else if ([menu_item_job isEqualToString:ACTION_QUIT]) {
					menu_item = [[NSMenuItem alloc] initWithTitle:menu_item_text action:@selector(quit:) keyEquivalent:@""];
				} else if ([menu_item_job isEqualToString:ACTION_EDIT_PREFS]) {
					menu_item = [[NSMenuItem alloc] initWithTitle:menu_item_text action:@selector(editPrefs:) keyEquivalent:@""];
				} else if ([menu_item_job isEqualToString:MENU_SEPARATOR]) {
					menu_item = [[NSMenuItem separatorItem] initWithTitle:@"" action:@selector(doNothing:) keyEquivalent:@""];
				} else {
					menu_item = [[NSMenuItem alloc] initWithTitle:menu_item_text action:@selector(doNothing:) keyEquivalent:@""];
				}
			} else {
				menu_item = [[NSMenuItem alloc] initWithTitle:menu_item_text action:@selector(doNothing:) keyEquivalent:@""];
			}

			if ([menu_item_indentation length]) {
				[menu_item setIndentationLevel:[menu_item_indentation intValue]];
			}

			[menu_item setTarget:self];
			[theMenu insertItem:menu_item atIndex:0];
		
		}
	}
	[statusItem setEnabled:YES];
}

/////////////////
// LOGOUT STUFF
- (IBAction)openLogoutWindow:(id)sender
{
	if ( ! loginWindowShowing ) {
		[logoutWindow setLevel:((NSScreenSaverWindowLevel - 1))];
		[logoutWindow center];
		[logoutWindow makeKeyAndOrderFront:nil];
		NSDictionary *scriptErr = nil;
		NSAppleScript *theScript = [[NSAppleScript alloc] initWithSource:[NSString stringWithFormat:@"activate"]];
		[theScript executeAndReturnError:&scriptErr];
		[theScript release];

		counter = timeout;
		[logoutTimerField setStringValue:[NSString stringWithFormat:@"Auto logout in %d seconds", counter]];
		logoutTimer = [[NSTimer scheduledTimerWithTimeInterval:1
														 target:self
													   selector:@selector(updateLogout)
													   userInfo:nil
														repeats:YES] retain];
		loginWindowShowing = YES;
	}
}

- (IBAction)logout:(id)sender
{
	[logoutButton setEnabled:NO];
	[logoutCancelButton setEnabled:NO];
    [logoutTimerField setStringValue:[NSString stringWithFormat:@"Now logging out", counter]];
	loggingout = YES;
}

-(void)updateLogout
{
	if (loggingout) {

		NSURL *scriptURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"logout" ofType:@"scpt"]];
		NSAppleScript *theScript = [[NSAppleScript alloc] initWithContentsOfURL:scriptURL error:nil];

		close(open(LOGOUT_TRIGGER_FILE, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH ));

		[theScript executeAndReturnError:nil];
		[theScript release];
		[logoutTimer invalidate];
		[logoutTimer release];
		[logoutWindow close];
		loggingout = YES;
		loginWindowShowing = NO;

	} else {

		counter--;
		[logoutTimerField setStringValue:[NSString stringWithFormat:@"Auto logout in %d seconds", counter]];
		if (counter == 0 ) {
			[self performSelector:@selector(logout:) withObject:nil afterDelay:0.0];
		}

	}
}

- (IBAction)cancelLogout:(id)sender
{
    [logoutTimer invalidate];
    [logoutTimer release];
    [logoutWindow close];
	loginWindowShowing = NO;
}

/////////////////
// RESET STUFF
- (IBAction)openResetWindow:(id)sender
{
	//    [resetWindow make Visialbe];
	if ( ! resetWindowShowing ) {
		[resetWindow setLevel:((NSScreenSaverWindowLevel - 1))];
		[resetWindow center];
		[resetWindow makeKeyAndOrderFront:nil];
		counter = timeout;
		[resetTimerField setStringValue:[NSString stringWithFormat:@"Auto reset in %d seconds", counter]];
		resetTimer = [[NSTimer scheduledTimerWithTimeInterval:1
														 target:self
													   selector:@selector(updateReset)
													   userInfo:nil
														repeats:YES] retain];
		resetWindowShowing = YES;
	}
}

- (IBAction)reset:(id)sender
{
	[resetButton setEnabled:NO];
	[resetCancelButton setEnabled:NO];
    [resetTimerField setStringValue:[NSString stringWithFormat:@"Now resetting", counter]];
	resetting = YES;
}

-(void)updateReset
{
	if (resetting) {
		close(open(RESET_TRIGGER_FILE, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH ));
		[resetTimer invalidate];
		[resetTimer release];
		[resetWindow close];
		resetting = NO;
		resetWindowShowing = NO;
		[resetButton setEnabled:YES];
		[resetCancelButton setEnabled:YES];
	} else {
		counter--;
		[resetTimerField setStringValue:[NSString stringWithFormat:@"Auto reset in %d seconds", counter]];
		if (counter == 0 ) {
			[self performSelector:@selector(reset:) withObject:nil afterDelay:0.0];
		}
	}
}

- (IBAction)cancelReset:(id)sender
{
    [resetTimer invalidate];
    [resetTimer release];
    [resetWindow close];
	resetWindowShowing = NO;
}





/////////////////
// QUIT STUFF

- (IBAction)quit:(id)sender
{
    [NSApp terminate:self];
}

/////////////////
// EDIT PREFS STUFF

- (IBAction)editPrefs:(id)sender
{
    [editPrefsWindow setLevel:((NSScreenSaverWindowLevel - 1))];
    [editPrefsWindow center];
    [editPrefsWindow makeKeyAndOrderFront:nil];
}

- (IBAction)cancelEditPrefs:(id)sender
{
	[editPrefsWindow close];
}

/////////////////
// Do nothing mon

- (IBAction)doNothing2:(id)sender
{
}


/////////////////

-(void)dealloc
{
    [statusItem release];
    [logoutTimer invalidate];
    [logoutTimer release];
    [resetTimer invalidate];
    [resetTimer release];
    [super dealloc];
}

@end
