#import "AppController.h"
#import <Security/Security.h>
#import <fcntl.h>
#import <sys/stat.h>
#import <unistd.h>

#define TIMEOUT (60)
#define RUN_MAINTENANCE_TRIGGER_FILE "/tmp/launch_maintenance"
#define LOGOUT_TRIGGER_FILE "/tmp/logoutNow"

@implementation AppController

- (void)quit
{
	exit(0);
}

- (BOOL)authorizeToRun
{
	OSStatus status;
	AuthorizationRef authorizationRef;
	status = AuthorizationCreate (NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
	if (status != errAuthorizationSuccess) {
		NSLog(@"Failed to create the authorization ref: %d.", status);
		return NO;
	}
	AuthorizationItem right = { kAuthorizationRightExecute, 0, NULL, 0 };
	AuthorizationRights rightSet = { 1, &right };
	AuthorizationFlags flags = kAuthorizationFlagDefaults |
		// kAuthorizationFlagPreAuthorize |
		kAuthorizationFlagInteractionAllowed |
		kAuthorizationFlagExtendRights;
	status = AuthorizationCopyRights (authorizationRef, &rightSet, kAuthorizationEmptyEnvironment, flags, NULL);
	return 	( status == errAuthorizationSuccess );
}

- (void)spawnQuitThread:(id)sender
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	sleep(TIMEOUT);
	if (!authorized) {
		exit(0);
	}
	[pool release];
}

- (void)awakeFromNib
{
	[NSThread detachNewThreadSelector:@selector(spawnQuitThread:) toTarget:self withObject:NULL];
	if ( [self authorizeToRun] ) {
		authorized = YES;
		[adminWindow setLevel:((NSScreenSaverWindowLevel - 1))];
		[adminWindow center];
		[adminWindow makeKeyAndOrderFront:nil];
		[NSTimer scheduledTimerWithTimeInterval:TIMEOUT target:self selector:@selector(quit) userInfo:nil repeats:NO];		
	} else {
		NSLog(@"Failed to authenticate.");
		[self quit];
	}
}

- (IBAction)logOut:(id)sender
{
	NSLog(@"Logging Out");
	NSURL *scriptURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"logout" ofType:@"scpt"]];
	NSAppleScript *theScript = [[NSAppleScript alloc] initWithContentsOfURL:scriptURL error:nil];

	close(open(LOGOUT_TRIGGER_FILE, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH ));

	[theScript executeAndReturnError:nil];
	[theScript release];
}

- (IBAction)restart:(id)sender
{
	NSLog(@"Restarting");
	NSURL *scriptURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"restart" ofType:@"scpt"]];
	NSAppleScript *theScript = [[NSAppleScript alloc] initWithContentsOfURL:scriptURL error:nil];

	[theScript executeAndReturnError:nil];
	[theScript release];
}

- (IBAction)runRadmind:(id)sender
{
	NSLog(@"Running Radmind");
	close(open(RUN_MAINTENANCE_TRIGGER_FILE, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH ));
	[self logOut:nil];
}

- (IBAction)shutdown:(id)sender
{
	NSLog(@"Shutting Down");
	NSURL *scriptURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"shutdown" ofType:@"scpt"]];
	NSAppleScript *theScript = [[NSAppleScript alloc] initWithContentsOfURL:scriptURL error:nil];
	[theScript executeAndReturnError:nil];
	[theScript release];
}

@end
