/*
Copyright (c) 2002 Regents of The University of Michigan.
All Rights Reserved.

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appears in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of The University
of Michigan not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission. This software is supplied as is without expressed or
implied warranties of any kind.

Research Systems Unix Group
The University of Michigan
c/o Wesley Craig
4251 Plymouth Road B1F2, #2600
Ann Arbor, MI 48105-2785

based loosely on the iHook source code
http://www.ihook.org
*/

#import "SPController.h"
#define SLIDETIME 2.0

static NSTimer	*myTimer = nil;
static int curFile;
static float pauseInSeconds;
static float defaultPauseInSeconds;
static BOOL firstView;

@implementation SPController

- ( id )init
{
    self = [super init];
    [NSApp setDelegate: self];
    files = [[NSMutableArray alloc] init];
    return( self );
}

- ( void )awakeFromNib
{
	// Create windows
	NSArray		*screens = [NSScreen screens];
	NSEnumerator *ee = [screens objectEnumerator];
	NSScreen *screen;

	windows = [[NSMutableArray alloc] initWithCapacity:1];
	while( screen = [ee nextObject] ) {

		NSRect windowRect = [screen frame];

		//windowRect = NSMakeRect([screen frame].origin.x, [screen frame].origin.y, [screen frame].size.width/2, [screen frame].size.height/2);
		
		SPWindow *myWindow = [[SPWindow alloc] initWithContentRect: windowRect
														 styleMask: NSBorderlessWindowMask
														   backing: NSBackingStoreBuffered
															 defer: NO];
		// Create subviews
		SPView *myView = [[SPView alloc] initWithFrame:[screen frame]];
		[myWindow setContentView: myView];

		// Window settings
		if ( [myWindow respondsToSelector:@selector(setCanBecomeVisibleWithoutLogin:)] ) {
			[myWindow setCanBecomeVisibleWithoutLogin:YES];
		}
		[myWindow setLevel: ( NSScreenSaverWindowLevel - 1 )];
		[windows addObject:myWindow];
	}

}

- ( void )applicationDidFinishLaunching: ( NSNotification * )aNotification
{
	// Read prefs
	NSString *picturesPref;
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addSuiteNamed:@"Xhooks/edu.utah.scl.logoutsaver"];

	[defaults registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:
		@"Pictures", @"picturesDir",
		@"7.0", @"slideTime",
		nil]];
	
	picturesPref = [defaults stringForKey:@"picturesDir"];
	defaultPauseInSeconds = [[defaults stringForKey:@"slideTime"] floatValue];

    // Load files
	int ii;
    NSArray *picArray;
    BOOL isDir;
    NSFileManager *manager;
	NSString *picturesDir;

	if ( [picturesPref isEqualToString:@"Pictures"] ) {
		picturesDir = [[NSBundle mainBundle] pathForResource: @"Pictures" ofType: nil];
	} else {
		picturesDir = picturesPref;
	}
	if ( picturesDir == nil ) {
		NSLog( @"Couldn't get pictures directory path!" );
		exit( 2 );
	}
    
    manager = [NSFileManager defaultManager];
    picArray = [manager directoryContentsAtPath: picturesDir];
                    
    for ( ii = 0; ii < [picArray count]; ii++ ) {
        NSString	*pPath = [NSString stringWithFormat: @"%@/%@",
                                    picturesDir, [picArray objectAtIndex: ii]];
                                    
        if ( [manager fileExistsAtPath: pPath isDirectory: &isDir]
                && ! isDir ) {
            //NSLog( @"%@", pPath );
            [files addObject: pPath];
        }
    }
    if ( [files count] <= 0 ) {
        NSLog( @"No files in %@. exiting.", picturesDir );
        exit( 2 );
    }

	// Bring windows front
	NSEnumerator *ee = [windows objectEnumerator];
	SPWindow *myWindow;
	while( myWindow = [ee nextObject] ) {
		SPView *myView = [myWindow contentView];
		[myWindow makeFirstResponder: myView];
		if ( [myWindow acceptsMouseMovedEvents] ) {
			NSLog( @"Accepts mouse moved" );
		}
		[myWindow makeKeyAndOrderFront: nil];
		if ( [myWindow respondsToSelector:@selector(orderFrontRegardless)] ) {
			[myWindow orderFrontRegardless];
		}
	}
    [NSApp activateIgnoringOtherApps: YES];
	[NSCursor hide];

	// Start slideshow
    curFile = 0;
	srandom( (unsigned int)time( NULL ) );
    [self changeBackground];
}

- ( void )changeBackground
{
    BOOL imageFileLoaded = NO;
    BOOL weblocFileLoaded = NO;
    BOOL movieFileLoaded = NO;

	NSImage	*imageFile = nil;
	QTMovie	*movieFile = nil;
	NSURL *webpage = nil;

	// Load file

    while ( ! imageFileLoaded || ! weblocFileLoaded || ! movieFileLoaded ) {
		if ( curFile >= [files count] ) curFile = 0;
		if ( [files count] <= 0 ) {
			NSLog( @"No valid pictures. exiting." );
			exit( 2 );
		}

		// Try webloc
		if ( [[[files objectAtIndex:curFile] pathExtension] isEqualToString:@"webloc"] ) {
			NSDictionary *dict = [NSDictionary dictionaryWithContentsOfURL:[NSURL fileURLWithPath:[files objectAtIndex:curFile]]];
			webpage = [NSURL URLWithString:[dict objectForKey:@"URL"]];
			weblocFileLoaded = YES;
			break;
		}

		// Try image
		imageFile = [[[NSImage alloc] initWithContentsOfFile: [files objectAtIndex:curFile]] autorelease];
		if ( imageFile != nil ) {
			imageFileLoaded = YES;
			break;
		}

		// Try movie
		if ( [QTMovie canInitWithFile:[files objectAtIndex:curFile]] ) {
			movieFile = [QTMovie movieWithFile:[files objectAtIndex:curFile] error:nil];
			movieFileLoaded = YES;
			break;
		}

		NSLog( @"Can't display file: %@", [files objectAtIndex:curFile] );
        [files removeObjectAtIndex: curFile];

    }

	// Display image/movie/webpage
	firstView = YES;
	NSEnumerator *ee = [windows objectEnumerator];
	SPWindow *myWindow;
	while( myWindow = [ee nextObject] ) {
		SPView *myView = [myWindow contentView];

		// Delete old views.
		NSEnumerator *ff = [[myView subviews] objectEnumerator];
		id unknownView;
		while( unknownView = [ff nextObject] ) {
			[unknownView removeFromSuperview];
			if ( [unknownView respondsToSelector:@selector(setImage:)] ) {
				[unknownView setImage: nil];
				[unknownView release];
			}
			if ( [unknownView respondsToSelector:@selector(setMovie:)] ) {
				[[unknownView movie] stop];
				[unknownView setMovie:nil];
				NSLog(@"%@",unknownView);
				[unknownView release]; // Crash and leak!
			}
			unknownView = nil;
		}

		// Show IMAGE
		if ( imageFileLoaded ) {
			
			NSSize mySize;
			mySize.width = [[imageFile bestRepresentationForDevice:nil] pixelsWide];
			mySize.height = [[imageFile bestRepresentationForDevice:nil] pixelsHigh];
			[imageFile setScalesWhenResized:YES];
			[imageFile setSize:mySize];
			
			NSImageView *myImageView = [[NSImageView alloc] initWithFrame:[myWindow frame]];
			[myView addSubview:myImageView];
			[myImageView setImage: imageFile];
			[myImageView setFrame: NSMakeRect(random() % (long) ([myWindow frame].size.width - mySize.width),
											  random() % (long) ([myWindow frame].size.height - mySize.height),
											  mySize.width,
											  mySize.height)];
											  
			pauseInSeconds = defaultPauseInSeconds;

			if ( firstView ) {
				// Reload
				myTimer = [NSTimer scheduledTimerWithTimeInterval: pauseInSeconds
									target: self
									selector: @selector( changeBackground )
									userInfo: nil
									repeats: NO];
			}
		}
		// Show MOVIE
		if ( firstView && movieFileLoaded ) {
			QTMovieView *myMovieView = [[QTMovieView alloc] initWithFrame:[myWindow frame]];
			[myView addSubview:myMovieView];
			[myMovieView setFillColor:[NSColor blackColor]];
			[myMovieView setControllerVisible:NO];
			[myMovieView setMovie:movieFile];
			[movieFile play];

			NSTimeInterval timeInterval;
			QTGetTimeInterval([movieFile duration], &timeInterval);
			pauseInSeconds = timeInterval;
			// Reload
			myTimer = [NSTimer scheduledTimerWithTimeInterval: pauseInSeconds
								target: self
								selector: @selector( changeBackground )
								userInfo: nil
								repeats: NO];
		}
		// Show WEBPAGE
		if ( firstView && weblocFileLoaded ) {
			WebView *myWebView = [[WebView alloc] initWithFrame:[myWindow frame]];
			[myView addSubview:myWebView];
			[[myWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:webpage]];
			[myWebView setFrameLoadDelegate:self]; // For webView:didFinishLoadForFrame
			[myWebView setHidden:YES];
			[webpage release];
		}

		if ( firstView ) {
			firstView = NO;
		}
		
		[myView setNeedsDisplay:YES];
	}

    curFile++;
}

- (void)webView:(WebView *)webView didFinishLoadForFrame:(WebFrame *)frame
{
    if (frame == [webView mainFrame]) {
		[webView setHidden:NO];
		pauseInSeconds = defaultPauseInSeconds;

		// Reload
		myTimer = [NSTimer scheduledTimerWithTimeInterval: pauseInSeconds
						target: self
						selector: @selector( changeBackground )
						userInfo: nil
						repeats: NO];
	}
}

- ( void )dealloc
{
    [files release];
    [windows release];
    [super dealloc];
}

@end
