#!/bin/sh

# This script assumes:
# - You are using radmind
# - You want to add custom X509 certs
#
# If you do (or need to do) those things, you have a problem, namely Apple regularly
# updates the certs in OS updates.  The certs are not stored seperate, but as one big
# file.  If you have custom overload of the cert file(s), your custom overloads will
# take precedence over your OS update overloads, thus you lose the Apples updates.
#
# This script automates customizing:
# 10.4 /System/Library/Keychains/X509Anchors and X509Certificates
# 10.5 /Library/Keychains/System.keychain
#
# This script was based on a script from the Mac OS X 10.4.7 updater and it is used
# as a post maintenace script with Xhooks to update the certs and anchors.

ANCHORS="/Library/Preferences/Xhooks/edu.utah.scl.certs/anchors"
CERTS="/Library/Preferences/Xhooks/edu.utah.scl.certs/certificates"

IFS="
"

osversion=`sw_vers -productVersion | sed 's/\([0-9]*\.[0-9]*\).*/\1/'`

if [ "$osversion" = "10.5" ]; then

	if [ `ls "$CERTS" | wc -l` -gt 0 ]; then
		for ii in `ls "$CERTS"`; do
			/usr/bin/security add-trusted-cert -d -k /Library/Keychains/System.keychain "$CERTS/$ii"	
		done

		if [ $? -gt 0 ];then
			echo "ERROR $0 - adding cert to /Library/Keychains/System.keychain" 1>&2
		fi
	fi

else

	if [ `ls "$ANCHORS" | wc -l` -gt 0 ]; then
		if [ ! -f "/System/Library/Keychains/X509Anchors" ]; then
			echo "ERROR $0 - /System/Library/Keychains/X509Anchors does not exist." 1>&2
			exit
		fi
	
		if [ "$xhooks_verbose" = "1" ]; then
			for ii in `ls "$ANCHORS"`; do
				echo "/usr/bin/security add-certificate -k /System/Library/Keychains/X509Anchors $ANCHORS/$ii"
			done

		fi
		
		for ii in `ls "$ANCHORS"`; do
				/usr/bin/security add-certificate -k /System/Library/Keychains/X509Anchors "$ANCHORS/$ii"
		done		
		
		if [ "$?" == 0 ]; then
			if [ "$xhooks_verbose" = "1" ]; then
				echo /bin/rm -rf "$ANCHORS/"*
			fi
			/bin/rm -rf "$ANCHORS/"*
		else
			echo "ERROR $0 - adding cert to /System/Library/Keychains/X509Anchors." 1>&2
			exit
		fi
	fi
	
	if [ `ls "$CERTS" | wc -l` -gt 0 ]; then
		if [ ! -f "/System/Library/Keychains/X509Certificates" ]; then
			echo "ERROR $0 - /System/Library/Keychains/X509Certificates does not exist."
			exit
		fi
	
		if [ "$xhooks_verbose" = "1" ]; then
			for ii in `ls "$CERTS"`; do
				echo "/usr/bin/security add-certificate -k /System/Library/Keychains/X509Certificates $CERTS/$ii"
			done
		fi

		for ii in `ls "$CERTS"`; do
			/usr/bin/security add-certificate -k /System/Library/Keychains/X509Certificates "$CERTS/$ii"
		done
		
		if [ "$?" == 0 ]; then
			if [ "$xhooks_verbose" = "1" ]; then
				echo /bin/rm -rf "$CERTS/"*
			fi
			/bin/rm -rf "$CERTS/"*
		else
			echo "ERROR $0 - adding cert to /System/Library/Keychains/X509Certificates." 1>&2
			exit
		fi
	fi

fi
