#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/logger.pl";

if ( -f "/Library/Preferences/Xhooks/edu.utah.scl.radmind_xhooks_conf.perl" ) {
  use lib "/Library/Preferences/Xhooks";
  require "edu.utah.scl.radmind_xhooks_conf.perl";
}

################################################################################
#
# This script changes /etc/hostconfig based on the contents of the array
# @HOSTCONFIG.  
#
################################################################################

my $new_file = "";
my %etc_hostconfig_hash;
my $resave = 0;

if ( ! nonempty_array ( @HOSTCONFIG ) ) {
  entman_logger ( "log", "ERROR: No Hostconfig items" );
  exit;
}

foreach my $item ( @HOSTCONFIG ) {
  if ( $item =~ /(.*)=(.*)/ ) {
    $etc_hostconfig_hash{$1}=$2;
  } else {
    entman_logger( "log", "Could not parse: $item" );
  }
}

if ( open INFILE, "< /etc/hostconfig" ) {
  while ( <INFILE> ) {
    chomp;
    if ( /(.*)=(.*)/ ) {
      my ( $key, $value ) = ($1, $2);
      if ( exists $etc_hostconfig_hash{$key} ) {
        if ( $etc_hostconfig_hash{$key} ne $value ) {
          my $new_value = $etc_hostconfig_hash{$key};
          $new_file .= "$key=$new_value\n";
          $resave = 1;
          delete $etc_hostconfig_hash{$key};
          entman_logger ( "log", "Changed $key to $new_value /etc/hostconfig." );
        } else {
          $new_file .= $_ . "\n";    
          delete $etc_hostconfig_hash{$key};
        }
      } else {
        $new_file .= $_ . "\n";    
      }
    } else {
      entman_logger ( "log", "Could not read line: $_" );
      $new_file .= $_ . "\n";    
    }
  }
  close ( INFILE );
} else {
  die "Could not read /etc/hostconfig: $!";
}

while ( my ( $key, $value ) = each %etc_hostconfig_hash ) {
  $new_file .= "$key=$value\n";
  entman_logger ( "log", "Added $key to $value /etc/hostconfig." );
  $resave = 1;
}

if ( $resave ) {
  if ( open OUTFILE, "> /etc/hostconfig" ) {
    print OUTFILE $new_file;
    close ( OUTFILE );
  } else {
    die "Could not write /etc/hostconfig: $!";
  }
  entman_logger ( "log drawer", "Resaved /etc/hostconfig." );
} else {
  entman_logger ( "log drawer", "No changes made to /etc/hostconfig." );
}

exit 0;
