#!/usr/bin/perl

################################################################################
# Copyright (c) 2009 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

my $location_of_ncutil = "/usr/sbin/ncutil"; # version 3.1.2 (required on 10.4)
my $networksetup = "/usr/sbin/networksetup"; # Only on 10.5

my $debug = 0;

################################################################################
# Set Proxy Settings
if ( get_short_os_version() eq "10.4" ) {
  if ( -e $location_of_ncutil ) {

    print "Setting proxy settings (10.4).\n" if $debug;

    system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "passive-ftp", "1" if true_var ( $PROXY_PASSIVE_FTP );

    if ( nonempty_var ( $HTTP_PROXY_SERVER ) and nonempty_var ( $HTTP_PROXY_PORT ) ) {
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "http-proxy-enable", "1";
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "http-proxy-host", $HTTP_PROXY_SERVER;
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "http-proxy-port", $HTTP_PROXY_PORT;
    } else {
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "http-proxy-enable";
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "http-proxy-host";
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "http-proxy-port";
    }

    if ( nonempty_var ( $HTTPS_PROXY_SERVER ) and nonempty_var ( $HTTPS_PROXY_PORT ) ) {
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "https-proxy-enable", "1";
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "https-proxy-host", $HTTPS_PROXY_SERVER;
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "https-proxy-port", $HTTPS_PROXY_PORT;
    } else {
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "https-proxy-enable";
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "https-proxy-host";
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "https-proxy-port";
    }

    if ( nonempty_array ( @PROXY_HOST_EXCEPTIONS ) ) {
      system $location_of_ncutil, "setprop", "/Automatic/Built-in Ethernet/Proxies", "host-exceptions", @PROXY_HOST_EXCEPTIONS;
    } else {
      system $location_of_ncutil, "destroyprop", "/Automatic/Built-in Ethernet/Proxies", "host-exceptions";
    }

    system $location_of_ncutil, "--apply-on-exit", "read", "/Automatic/Built-in Ethernet/Proxies";

  } else {
    print "ERROR: $location_of_ncutil is missing.\n";
  }

} else {

  if ( -e $networksetup ) {

    print "Setting proxy settings (10.5).\n" if $debug;

    my @bla = `networksetup listnetworkserviceorder | grep -b1 en0`;
    $bla[0] =~ s/^\d+-\(\d+\) //;
    chomp $bla[0];
    my $service_name = $bla[0];
    print "Service Name: $service_name\n" if $debug;

    print "$networksetup -setpassiveftp $service_name $PROXY_PASSIVE_FTP\n" if nonempty_var ( $PROXY_PASSIVE_FTP ) and $debug;;
    system $networksetup, "-setpassiveftp", $service_name, $PROXY_PASSIVE_FTP if nonempty_var ( $PROXY_PASSIVE_FTP );

    if ( nonempty_var ( $HTTP_PROXY_SERVER ) and nonempty_var ( $HTTP_PROXY_PORT ) ) {
      print "$networksetup setwebproxy $service_name $HTTP_PROXY_SERVER, $HTTP_PROXY_PORT\n" if $debug;
      print "$networksetup setwebproxystate $service_name on\n" if $debug;
      system $networksetup, "setwebproxy", $service_name, $HTTP_PROXY_SERVER, $HTTP_PROXY_PORT;
      system $networksetup, "setwebproxystate", $service_name, "on";
    } else {
      print "$networksetup setwebproxy $service_name\n" if $debug;
      print "$networksetup setwebproxystate $service_name off\n" if $debug;
      system $networksetup, "setwebproxy", $service_name, "", "";
      system $networksetup, "setwebproxystate", $service_name, "off";
    }

    if ( nonempty_var ( $HTTPS_PROXY_SERVER ) and nonempty_var ( $HTTPS_PROXY_PORT ) ) {
      print "$networksetup setsecurewebproxy $service_name $HTTPS_PROXY_SERVER, $HTTPS_PROXY_PORT\n" if $debug;
      print "$networksetup setsecurewebproxystate $service_name on\n" if $debug;
      system $networksetup, "setsecurewebproxy", $service_name, $HTTPS_PROXY_SERVER, $HTTPS_PROXY_PORT;
      system $networksetup, "setsecurewebproxystate", $service_name, "on";
    } else {
      print "$networksetup setsecurewebproxy $service_name\n" if $debug;
      print "$networksetup setsecurewebproxystate $service_name off\n" if $debug;
      system $networksetup, "setsecurewebproxy", $service_name, "", "";
      system $networksetup, "setsecurewebproxystate", $service_name, "off";
    }

    if ( nonempty_array ( @PROXY_HOST_EXCEPTIONS ) ) {
      print "$networksetup setproxybypassdomains $service_name @PROXY_HOST_EXCEPTIONS\n" if $debug;
      system $networksetup, "setproxybypassdomains", $service_name, @PROXY_HOST_EXCEPTIONS;
    } else {
      print "$networksetup setproxybypassdomains $service_name Empty\n" if $debug;
      system $networksetup, "setproxybypassdomains", $service_name, "Empty";
    }

  } else {
    print "ERROR: $networksetup is missing.\n";
  }

}
