#!/usr/bin/perl

use Foundation;
use lib "/Library/Xhooks/Modules";
require "xhooks/lib/perlplist.pl";
require "xhooks/lib/common.pl";

if ( -f "/Library/Preferences/Xhooks/edu.utah.scl.radmind_xhooks_conf.perl" ) {
  use lib "/Library/Preferences/Xhooks";
  require "edu.utah.scl.radmind_xhooks_conf.perl";
}

my $debug_level = ( defined $ARGV[0] ) ? $ARGV[0] : 0;

if ( defined @SUPER_DEFAULTS and $#SUPER_DEFAULTS >= 0 ) {

  foreach $super_default ( @SUPER_DEFAULTS ) {

    print "SED_superDefaults.pl: $super_default\n" if $debug_level > 0;

    my @params = split_string_with_quotes( $super_default );
    
    my $filename = shift @params;
    my $action = shift @params;
    
    print "Super Default file: $filename\n" if $debug_level > 1;
    print "Super Default params: @params\n" if $debug_level > 1;

    if ( $action eq "--add" ) {

      if ( $#params % 2 != 0 ) {

        my $dict;
        if ( -e $filename ) {
          $dict = loadDefaults( $filename );
        } else {
      	  print "The file $filename does not exist, creating a new file.\n";
      	  $dict = NSMutableDictionary->dictionary();
        }
        setPlistObjectForce( $dict, \@params );
        printObject ($dict) if $debug_level > 1;
        saveDefaults( $dict, $filename );

      } else {
        print "ERROR: SED_superDefaults.pl, wrong number of parameters for --add: @params\n";
      }
    } elsif ( $action eq "--delete" ) {
      
      #if ( $#params % 2 != 0 ) {

        if ( -e $filename ) {
          $dict = loadDefaults( $filename );
          if ( removePlistObject($dict, @params) ) {

            printObject ($dict) if $debug_level > 1;
            saveDefaults( $dict, $filename );

          } else {
            print "Removing @params from $filename failed (maybe it wasn't there to begin with).\n";
          }
        } else {
          print "The file $filename does not exist, nothing to delete!\n";
        }
      #} else {
      #  print "ERROR: SED_superDefaults.pl, wrong number of parameters for --delete: @params\n";
      #}
    } elsif ( $action eq "--read" ) {
      print "Super Default --read not supported yet\n";
    } else {
      print "Super Default unknown action: $action\n";
    }
    
    print "\n" if $debug_level > 1;
  }
}
