#!/usr/bin/perl

################################################################################
# Copyright (c) 2009 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

#
# This script modifies or creates the file /Library/Preferences/com.apple.SystemLoginItems.plist
# if the array @SYSTEM_LOGIN_ITEMS has values, which should be paths to apps that are launched
# at the loginwindow or something, Timbuktu and the Wacom tablet driver are the ones I know of:
#
## SYSTEM_LOGIN_ITEMS += /Library/StartupItems/Tablet/TabletDriver.app
## SYSTEM_LOGIN_ITEMS += /Applications/Timbuktu Pro.app/Contents/SharedSupport/Timbuktu Host.app
#

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
use Foundation;
require "xhooks/lib/perlplist.pl";

if ( ! nonempty_array ( @SYSTEM_LOGIN_ITEMS ) ) {
  exit 0;
}

my $message = "";
my $system_login_items_file = "/Library/Preferences/com.apple.SystemLoginItems.plist";

%appHash=();
if ( -e $system_login_items_file ) {
  $dict = loadDefaults( $system_login_items_file );
  @appArray = perlArrayFromNSArray( getPlistObject( $dict, "AutoLaunchedApplicationDictionary" ), 1 );
  foreach $jj ( @appArray ) {
	$appHash{$$jj{"Path"}}=1;
  }
  $message = "Modified";
} else {
  $dict = NSMutableDictionary->dictionary();
  @appArray = ();
  $message = "Created";
}

foreach $ii ( @SYSTEM_LOGIN_ITEMS ) {
  if (! $appHash{$ii}) {
	push @appArray, { "Path" => $ii };
  }
}

setPlistObject( $dict, "AutoLaunchedApplicationDictionary", cocoaArrayFromPerlArray( @appArray ) );
saveDefaults( $dict, $system_login_items_file );

system "/usr/bin/plutil", "-convert", "binary1", $system_login_items_file;

print "$message $system_login_items_file\n";

exit 0;
