#!/bin/sh
#
# By Peter Brezny
# Modifications done to support dynamic IP and default OS X configuration
# by D. Cote, available at: http://www.novajo.ca/firewall.html
#
# Chopped up by The University of Utah Student Computer Labs for Xhooks
#
# Simple stateful network firewall rules for IPFW with NAT v. 1.01
# See bottom of file for instructions and description of rules
# Created 20001206206 by Peter Brezny, pbrezny@purplecat.net (with a great
# deal of help from freebsd-security@freebsd.org).  Specific questions
# about the use of ipfw should be directed to freebsd-ipfw@freebsd.org or
# more general security questions to freebsd-security@freebsd.org.
# Use this script at your own risk.
#
# if you don't know the a.b.c.0/xx notation for ip networks the ipsubnet
# calculator can help you. /usr/ports/net/ipsc-0.4.2
#

if [ -e '/Library/Preferences/Xhooks/edu.utah.scl.radmind_xhooks_conf.bash' ] ; then
	. /Library/Preferences/Xhooks/edu.utah.scl.radmind_xhooks_conf.bash
fi

# Make sure logging is enabled (disabled by default)
if [ `/usr/sbin/sysctl -n net.inet.ip.fw.verbose` == 0 ] ; then
        /usr/sbin/sysctl -w net.inet.ip.fw.verbose=1
fi

#
# Define your variables
#
fwcmd="/sbin/ipfw"      # leave as is if using ipfw
oif="en0"
quiet="-q"

#
# End of required user input if you only intend to allow ssh connections to
# this box from the outside. If other services are required, edit line 96
# as necessary.
#
###
# Rules with descriptions
#
# Basic rules: there is no need to modify anything in this first section.
# This is the bare minimum to block simple spoofing.
###
#
# Force a flush of the current firewall rules before we reload
$fwcmd $quiet -f flush

# Allow your loop back to work
$fwcmd $quiet add allow all from any to any via lo0

# Prevent spoofing of your loopback
$fwcmd $quiet add deny log all from any to 127.0.0.0/8

# Stop private networks (RFC1918) from entering the outside interface.
$fwcmd $quiet add deny log ip from 192.168.0.0/16 to any in via $oif
$fwcmd $quiet add deny log ip from 172.16.0.0/12 to any in via $oif
$fwcmd $quiet add deny log ip from 10.0.0.0/8 to any in via $oif
$fwcmd $quiet add deny log ip from any to 192.168.0.0/16 in via $oif
$fwcmd $quiet add deny log ip from any to 172.16.0.0/12 in via $oif
$fwcmd $quiet add deny log ip from any to 10.0.0.0/8 in via $oif

###
# Xhooks Customized rules
###
IFS="
"

for ii in ${FIREWALL_RULES[@]}
do
	ii="$quiet $ii"
 	ipfw $ii
done

# Allow everything else
$fwcmd $quiet add 65535 allow ip from any to any
