#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

# Activates/deactivates permissions on extra hard disks

$debug_this_script = 0; # This script might need debugging if the output of disktool changes with some OS update

@DEACTIVATE_VOLUME_PERMISSIONS = ();
@ACTIVATE_VOLUME_PERMISSIONS = ();

if ( -f "/Library/Preferences/Xhooks/edu.utah.scl.radmind_xhooks_conf.perl" ) {
	use lib "/Library/Preferences/Xhooks";
	require "edu.utah.scl.radmind_xhooks_conf.perl";
} else {
	die "ERROR $0 - The file /Library/Preferences/Xhooks/edu.utah.scl.radmind_xhooks_conf.perl is missing and this script requires it!\n";
}

if ( ! defined @DEACTIVATE_VOLUME_PERMISSIONS and ! defined @ACTIVATE_VOLUME_PERMISSIONS ) {
	exit;
}

my %deactivate_volumes_perms = ();
my %activate_volumes_perms = ();

if ( defined @DEACTIVATE_VOLUME_PERMISSIONS ) {
	foreach ( @DEACTIVATE_VOLUME_PERMISSIONS ) {
		print "Deactivate '$_'\n" if $debug_this_script;
		$deactivate_volumes_perms{$_} = 1;
	}
}
if ( defined @ACTIVATE_VOLUME_PERMISSIONS ) {
	foreach ( @ACTIVATE_VOLUME_PERMISSIONS ) {
		print "Activate '$_'\n" if $debug_this_script;
		$activate_volumes_perms{$_} = 1;
	}
}

my @volumes = `disktool -l`;
foreach my $line ( @volumes ) {

	print $line if $debug_this_script;
	my @lineparts = split ',', $line;

	# This next 2 lines might need future debugging if the output of disktool -l changes with an OS update
	if ( index ($lineparts[1],"Mountpoint = '/Volume") == 0 and $lineparts[2] eq ' fsType = \'hfs\'' ) {
		if ( $lineparts[3] =~ / volName = '(.*)'\)/ ) {

			if ( $1 ne "" and -d "/Volumes/$1" ) {

				# the volume exists and is ignorable
				print "Ignorable and exists: $1\n" if $debug_this_script;

				if ( defined $deactivate_volumes_perms{$1} and $deactivate_volumes_perms{$1} ) {
					print "Deactivating: /usr/sbin/vsdbutil -d /Volumes/$1\n" if debug_this_script;
					print "$0 - Deactivating: /usr/sbin/vsdbutil -d /Volumes/$1\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});
					system "/usr/sbin/vsdbutil", "-d", "/Volumes/$1";
				}

				if ( defined $activate_volumes_perms{$1} and $activate_volumes_perms{$1} ) {
					print "Activating: /usr/sbin/vsdbutil -a /Volumes/$1\n" if debug_this_script;
					print "$0 - Activating: /usr/sbin/vsdbutil -a /Volumes/$1\n" if ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'});
					system "/usr/sbin/vsdbutil -a \"/Volumes/$1\" ";
				}

			}
		}
	}
}

exit 0;
