#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

################################################################################
# Create Library/Preferences/loginwindow.plist (for login startup items)
#

if ( defined @GLOBAL_LOGINWINDOW_PLIST and $#GLOBAL_LOGINWINDOW_PLIST >= 0 ) {
  write_loginitems_file( "/", @GLOBAL_LOGINWINDOW_PLIST );
} else {
  entman_logger ( "log drawer", "INFO: GLOBAL_LOGINWINDOW_PLIST undefined." );
}

if ( defined @TEMPLATE_USER_LOGINWINDOW_PLIST and $#TEMPLATE_USER_LOGINWINDOW_PLIST >= 0 ) {
  write_loginitems_file( "$template_homefolder/", @TEMPLATE_USER_LOGINWINDOW_PLIST );
} else {
  entman_logger ( "log drawer", "INFO: TEMPLATE_USER_LOGINWINDOW_PLIST undefined." );
}

################################################################################
# Subroutines
sub write_loginitems_file {
  my ( $loginitems_file_path, @login_startup_items ) = @_;

  if ( ! -d "${loginitems_file_path}Library/Preferences" ) {
    entman_logger ( "log drawer", "The folder ${loginitems_file_path}Library/Preferences does not exist." );
    return;
  }

  if ( $#login_startup_items < 0 ) {
    entman_logger ( "log drawer", "There are no login items to write." );
    return;
  }

  my $loginfile = "${loginitems_file_path}Library/Preferences/loginwindow.plist";

  entman_logger ( "log drawer", "Modding $loginfile." );

  open ( FILEHANDLE, ">$loginfile" );

  print FILEHANDLE   '<?xml version="1.0" encoding="UTF-8"?>'."\n";
  print FILEHANDLE   '<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">'."\n";
  print FILEHANDLE   '<plist version="1.0">'."\n";
  print FILEHANDLE   '<dict>'."\n";
  print FILEHANDLE   '        <key>AutoLaunchedApplicationDictionary</key>'."\n";
  print FILEHANDLE   '        <array>'."\n";
  # List of auto launch items.

  foreach $login_start_item_string ( @login_startup_items ) {
    if ( $login_start_item_string =~ /(.*) (\d)\s*$/ ) {
	  my $login_startup_item_path = $1;
	  my $login_start_item_hidden = $2;
	  print FILEHANDLE "                <dict>\n";
	  print FILEHANDLE "                        <key>Hide</key>\n";
	  if ( $login_start_item_hidden == 1 ) {
		print FILEHANDLE "                        <true/>\n";
	  } else {
		print FILEHANDLE "                        <false/>\n";
	  }
	  print FILEHANDLE "                        <key>Path</key>\n";
	  print FILEHANDLE "                        <string>$login_startup_item_path</string>\n";
	  print FILEHANDLE "                </dict>\n";
    }
  }

  print FILEHANDLE   '        </array>'."\n";
  print FILEHANDLE   '</dict>'."\n";
  print FILEHANDLE   '</plist>'."\n";

  close ( FILEHANDLE );

  system "/usr/bin/plutil -convert binary1 \"$loginfile\"";
}
