#!/usr/bin/perl

################################################################################
# create_cached_home_folders.pl
#
# This script creates home folders backups.  It requires a parameter that says
# what the minimum home folders should be.  If the minimum exists, this script
# does nothing.  If the minimum does not exist.  This script will create ONE,
# only one.  Keep calling this script to get the minimum built up.  When the
# minimum is reached, this script will do nothing.
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

use Fcntl qw/O_WRONLY O_CREAT O_EXCL/;

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
require "edu.utah.scl.home_folder/lib/home_folder_lib.pl";

$tmp_home_folder = "/Library/Preferences/Xhooks/Caches/edu.utah.scl.home_folder/newTempHome";
my $homefolderQuitTriggerFile = "/Library/Preferences/Xhooks/triggerfiles/creatingHomeFolders";
my $assimilateWaitTriggerFile = "/Library/Preferences/Xhooks/triggerfiles/movingHomeFolders";

my $debug = 0;
$debug = 1 if defined $ARGV[1];

################################################################################
# Go
#
print "Started." if $debug;
$rightnow = time;

##
# Quit if another one of me is running. The following scripts call this one, and
# conflicts could occur.  This really sucks and needs to be improved...
##
my $wait = 1;
my $counter = 1;
while ( $wait and $counter < 10 ) {
  print "Waiting for assimilate trigger file: $homefolderQuitTriggerFile...\n" if $debug;
  $wait = 0;
  $counter++;
  if ( open(FH, "<", $homefolderQuitTriggerFile) ) {
	print "Trigger exists: $homefolderQuitTriggerFile\n" if $debug;
	$wait = 1;
	close(FH);
	sleep 1;
  } else {
	sysopen(FH, $homefolderQuitTriggerFile, O_WRONLY | O_CREAT | O_EXCL) or $wait = 2;
  }
  if ( $wait == 2 ) {
	print "Race condition averted!\n";
	sleep 1;
  }
}
if ( $wait ) {
  print "Quiting because another one of me is running.";
  @other_processes = `ps -awwx`;
  foreach $line ( @other_processes ) {
    chomp $line;
    print "$line";
  }
  
  sleep 1;
  exit 1;
}

#############################################
# The minimum number of folders
if ( ! defined $ARGV[0] ) {
  print "Please specify how many home folders should exist.\n";
  close(FH);
  unlink $homefolderQuitTriggerFile;
  exit 1;
}
$minFolders = $ARGV[0];

################################################################################
# Duplicate all the user folders

#debug:
#$test = count_home_folder_cache_folders();
#entman_logger ( "log", "Current number of folders $test ; min folders $minFolders " );

$number_of_folders_created = 0;

while ( count_home_folder_cache_folders() < $minFolders ) { 

  #######################
  # remove any folder that may already be in /tmp that has the desired name.
  if ( -e "$tmp_home_folder" ) {
    system "chflags -R nouchg $tmp_home_folder";
    system "rm -r $tmp_home_folder";
  }

  #######################
  create_home_folder ( "$tmp_home_folder" );

  #######################
  # move the folder

  # Wait, just in case home folders are being moved.
  my $wait = 1;
  my $counter = 1;
  while ( $wait and $counter < 100 ) {
    print "Waiting for assimilate trigger file: $assimilateWaitTriggerFile...\n" if $debug;
    $wait = 0;
    $counter++;
    if ( open(FH, "<", $assimilateWaitTriggerFile) ) {
      print "Trigger exists: $assimilateWaitTriggerFile\n" if $debug;
      $wait = 1;
      close(FH);
      sleep 1;
    } else {
      sysopen(FH, $assimilateWaitTriggerFile, O_WRONLY | O_CREAT | O_EXCL) or $wait = 2;
    }
    if ( $wait == 2 ) {
      print "Race condition averted!\n";
      sleep 1;
    }
  }
  if ( $counter > 99 ) {
      print "Error, error, error!  No home folder cache!  Help me, I'm dying...";
      close(FH);
      unlink $homefolderQuitTriggerFile;
      exit 1;
  }

  # Find out the next folder in line
  $createMe=count_home_folder_cache_folders() + 1;

  # if, for some reason, there is a mess up in this script and the folders 
  # are not in order, remove them all and just create one...
  if ( -e "$homeFolderBackups/$createMe" ) {
    print "Oops, the homefolderbackup is out of order.  Deleting all and only going to do 1 folder.";
    system "chflags -R nouchg \"$homeFolderBackups/\"";
    system "rm -r \"$homeFolderBackups/\"";
    system "mkdir \"$homeFolderBackups/\"";
    $createMe = 1;
    $minFolders = 1;
  }

  # now move it (this way the folder is completely copied)
  system "mv \"$tmp_home_folder\" \"$homeFolderBackups/$createMe\"";

  $number_of_folders_created++;

  # If, for some reason, there are not enough home folders, then just delete 
  # them all and start over.
  if ( ! -e "$homeFolderBackups/1" ) {
    print "Oops, the homefolderbackup is out of order.  Deleting all and only going to do 1 folder.";
    system "chflags -R nouchg \"$homeFolderBackups/\"";
    system "rm -r \"$homeFolderBackups/\"";
    system "mkdir \"$homeFolderBackups/\"";
    $minFolders = 1;
  }

  close(FH);
  unlink $assimilateWaitTriggerFile;
}

################################################################################
# all done

$totaltime = time - $rightnow;
print "Done. Took $totaltime, created $number_of_folders_created." if $debug;

close(FH);
unlink $homefolderQuitTriggerFile;

exit 0;
