#!/usr/bin/perl

################################################################################
# create_lost_and_found.pl
#
# The purpose of this script is to cleanup the last user home folder (/Users/<name>).
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

################################################################################
# This script deals with the old home folder (ex. /Users/guest).
#          On kiosks, the home folder is deleted.  On other computers the old 
# home folder (ex /Users/guest) is moved to the "lost and found".  The owner of 
# the old home folder is found from the file ~/.username, which is created by 
# this script (search for $login_name_trigger below).  The lost and found folder 
# is named after the owner (ex. 
# /Library/Preferences/Xhooks/Caches/edu.utah.scl.home_folder/lost_and_found/joe1234).
#          And, it does this really bad cleanup job on the folder that was 
# just moved to /Library/Preferences/Xhooks/Caches/edu.utah.scl.home_folder/lost_and_found
# (this code needs to be  improved, and there is untested code already to do this,
# but we haven't had time to test it--and I think I lost the code... it was years ago).
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
require "edu.utah.scl.home_folder/lib/home_folder_lib.pl";

use File::Find;

$template_homefolder_files = "/Library/Preferences/Xhooks/Caches/edu.utah.scl.home_folder/home_folder_files";
$homefolder_lostandfound = "Desktop/Lost & Found";

my $print_debug = 1;

#$lostAndFoundDays = "14"; # Number of days items will be saved in a lost and found - NOT IMPLEMENTED YET!!!

################################################################################
# GO:
my $rightnow = time;
entman_logger ( "log", "Started..." );

my $theHomeFolder = $userHomeFolder;

print "$theHomeFolder\n" if $print_debug;

if ( -e $theHomeFolder ) {  # $theHomeFolder ex. /Users/student

  print "$theHomeFolder exists\n" if $print_debug;

  # Get the username of the last user
  if ( -e "$theHomeFolder/$login_name_trigger" ) {
    
    print "$theHomeFolder/$login_name_trigger exists\n" if $print_debug;
    
    # Keep a lost and found
    $lastLoginTimeStamp = getTimeStamp( getFileModTime( "$theHomeFolder/$login_name_trigger" ) ); # /Users/student/.username
    $lastUsername = readOneLineFile( "$theHomeFolder/$login_name_trigger" ); # ex "abc1234" (typical university id... no?)

    # Create the hidden lost and found folder for this user
    if ( ! -e "$global_home_folder_lost_and_found/$lastUsername" ) {
      print "mkdir \"$global_home_folder_lost_and_found/$lastUsername\"\n" if $print_debug;
      system "mkdir \"$global_home_folder_lost_and_found/$lastUsername\"";
      print "/usr/sbin/chown -R $lastUsername \"$global_home_folder_lost_and_found/$lastUsername\"\n" if $print_debug;
      system "/usr/sbin/chown -R $lastUsername \"$global_home_folder_lost_and_found/$lastUsername\"";
    }

    # create location for lost and found
    #
    # Note, in 10.5, the system keeps track of all the dirs, so instead of moving the contents of the home folder,
    # we got to move the content of the home dirs.  That is why there is so much crapping code below.
    #
    my $newpath = "$global_home_folder_lost_and_found/$lastUsername/$lastLoginTimeStamp";
    print "$newpath\n" if $print_debug;
    handle_stupid_homedir_path ( "$newpath/Desktop", "$theHomeFolder/Desktop", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Documents", "$theHomeFolder/Documents", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Downloads", "$theHomeFolder/Downloads", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Library", "$theHomeFolder/Library", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Movies", "$theHomeFolder/Movies", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Music", "$theHomeFolder/Music", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Pictures", "$theHomeFolder/Pictures", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Public", "$theHomeFolder/Public", $lastUsername );
    handle_stupid_homedir_path ( "$newpath/Sites", "$theHomeFolder/Sites", $lastUsername );
    handle_stupid_homedir_path ( "$newpath", "$theHomeFolder", $lastUsername );

    if ( ! -e "$newpath/.dontcleanup" ) {
      # Cleanup old home folder
      cleanup_old_home_folder ( "$global_home_folder_lost_and_found/$lastUsername", $lastLoginTimeStamp );
    }

  } else {

    # This will happen if the system makes a home folder.  Which shouldn't happen.
    entman_logger ( "log", "Alert, $theHomeFolder/$login_name_trigger missing.  Deleting home folder." );

    print "deleting $theHomeFolder\n" if $print_debug;

    # Delete folder
    put_in_trash( $theHomeFolder );

  }

}

$totaltime = time - $rightnow;
entman_logger ( "log", "Done.  Took $totaltime." );

exit 0;

sub handle_stupid_homedir_path {
  my ( $newdir, $olddir, $lastUsername ) = @_;
  if ( -d $olddir ) {

    print "/bin/mkdir -p \"$newdir\"\n" if $print_debug;
    system "/bin/mkdir", "-p", "$newdir";# or print "Couldn't create $newdir $!\n";

    # Move the contents of the old folder to the hidden lost and found
    print "/bin/mv \"$olddir\"/* \"$newdir\"\n" if $print_debug;
    system "/bin/mv \"$olddir\"/* \"$newdir\"";
  
    # Move the contents of the old folder to the hidden lost and found
    print "/bin/mv \"$olddir\"/.* \"$newdir\"\n" if $print_debug;
    system "/bin/mv \"$olddir\"/.* \"$newdir\"";

	# Chown new
    system "/usr/sbin/chown", $lastUsername, $newdir;

    # Remove old
    system "/bin/rmdir", $olddir;
  } else {
    print "The dir $olddir is missing and well that defeats the whole point of handling it......";
  }
}

sub delete_folder {
  my ( $my_folder ) = @_;

  print "chflags -R nouchg \"$my_folder\"\n" if $print_debug;
  system "chflags -R nouchg \"$my_folder\"";
  print "rm -rf -- \"$my_folder\"\n" if $print_debug;
  system "rm -rf -- \"$my_folder\"";
  
}

####################################
# STEP 4 - DEAL WITH LOST AND FOUND
# If this doesn't work, it is probably because you are using special characters 
# (other than " " or "&", which this script will fix).

#
#  THIS SUBROUTINE NEEDS SERIOUS FIXING UP
#

sub cleanup_old_home_folder {
  my ( $user_lost_and_found, $my_folder ) = @_;
  my $path_to_home_folder = "$user_lost_and_found/$my_folder";

  ####################################
  # Move any old timestamped Lost and Found folders out of Desktop/Lost & Found up a level
  print "do i exist $path_to_home_folder/$homefolder_lostandfound\n" if $print_debug;
  if ( -e "$path_to_home_folder/$homefolder_lostandfound" ) { # "~/Desktop/Lost & Found";
    # mv ~/Desktop/Lost\ \&\ Found/* ~/..;
    @dirs = getFileFolderList( "$path_to_home_folder/$homefolder_lostandfound/" );
    foreach $i ( @dirs ) {
      print "/bin/mv \"$path_to_home_folder/$homefolder_lostandfound/$i\" \"$user_lost_and_found/\"\n" if $print_debug;
      system "/bin/mv \"$path_to_home_folder/$homefolder_lostandfound/$i\" \"$user_lost_and_found/\"";
    }
    system "/bin/rm -rf \"$path_to_home_folder/$homefolder_lostandfound\""; # rm -rf ~/Desktop/Lost\ \&\ Found
  }

  ####################################
  # CLEAN UP THE OLD HOME FOLDER

  # Delete Known throw away folders
  #if ( not test box!! ) {
    @home_folder_auto_delete_these_folders = ("Library", "Shared/Library", "SorensonMedia/Squeeze", );
    foreach $auto_detele ( @home_folder_auto_delete_these_folders ) {
      put_in_trash( "$path_to_home_folder/$auto_detele" );
    }
  #}

  ####################################
  # Everything else

  if ( chdir $path_to_home_folder ) {

  if ( open FILE, "< $template_homefolder_files" ) {
      while ( <FILE> ) {
        my $line = $_;
        my @parts = split / /, $line;
        my $path = unescape_radmind_path ( $parts[0] );
        my $test = 1;
        my $index;
        foreach $auto_detele ( @home_folder_auto_delete_these_folders ) {
          $index = index $path, $auto_detele;
          if ( $index > 0 and $index < 3 ) { $test = 0; last; }
        }
        if ( $test and -e $path ) {
          if ( ! unlink $path ) {
            system "/usr/bin/chflags -R nouchg \"$path\"";
            unlink $path or entman_logger ("log", "Couldn't unlink $path\n");# if $print_debug;
          }
        }
      }
      close FILE;
    } else {
      entman_logger ("log", "Can't open $template_homefolder_files: $!");
      return;
    }
  
    # Remove Finder created files (.DS_Store, .localized) # -type l -or 
    system "/usr/bin/find ./ \\( -name \".DS_Store\" -or -name \".localized\" \\) -delete";
  
    # Remove empty directories
    finddepth(sub{rmdir},'.');
  
    # cleanup Trash, make visible if stuff is in it
    if ( -e "$path_to_home_folder/.Trash" ) {
      system "/bin/mv \"$path_to_home_folder/.Trash\" \"$path_to_home_folder/Trash\"";
      system "/usr/local/bin/SetFile", "-a", "v", "$path_to_home_folder/Trash";
    }
  
    # cleanup root folder
    system "/bin/rm -f \"$path_to_home_folder/$login_name_trigger\"";
    if ( findNumberOfFoldersAndFiles( "$path_to_home_folder" ) < 1 ) {
      system "/bin/rm -rf \"$path_to_home_folder\"";
    }
  
    # If there is nothing in the lost and found, well, just delete the whole thing.
    if ( findNumberOfFoldersAndFiles( "$user_lost_and_found" ) < 1 ) {
      system "/bin/rm -rf \"$user_lost_and_found\"";
    }
  } else {
    entman_logger( "log", "Couldn't chdir $path_to_home_folder: $!");
  }
}

  ####################################
  # DO GENERAL LOST AND FOUND CLEANUP

  # remove folders that are too old
  #@dirs = getFileFolderList( "/Users" );
  #foreach $i ( @dirs ) {
  #  if ( -M "$global_home_folder_lost_and_found/$i" > $lostAndFoundDays ) {
  #    system "/bin/rm -rf \"$global_home_folder_lost_and_found/$i\"";
  #  }
  #}

  # remove folders that are too big
  #@dirs = getFileFolderList( "/Users" );
  #foreach $i ( @dirs ) {
  #  if ( -M "$global_home_folder_lost_and_found/$i" > $lostAndFoundDays ) {
  #    #print "/bin/rm -rf \"$global_home_folder_lost_and_found/$i\"";
  #  }
  #}
