#!/usr/bin/perl

################################################################################
#
# Copyright (c) 2009 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

use lib "/Library/Xhooks/Modules";
require "xhooks/lib/config.pl";

if ( -e "/Library/Xhooks/Modules/edu.utah.scl.display/display_conf.pl" ) {
  require "edu.utah.scl.display/display_conf.pl";
}

if ( nonempty_var ( $KIOSK_OPEN_NEW_BROWSER_WINDOW_DELAY ) ) {
  $KIOSK_OPEN_NEW_BROWSER_WINDOW_DELAY = 5;
}

my $timeTillNextCheck = $KIOSK_OPEN_NEW_BROWSER_WINDOW_DELAY; # wait this many seconds before iterating the main event loop

my $debug = ( defined $ARGV[0] ) ? 1 : 0;
my $logout_trigger_file = "/tmp/logoutNow";
my $run_startup_post_maintenance = "/Library/Preferences/Xhooks/triggerfiles/run_startup_post_maintenance";
my $radmind_finished = "/Library/Preferences/Xhooks/triggerfiles/radmind_finished";
my $wait_for_radmind = "/var/log/wait_for_radmind";
my $make_browser_window_big_trigger = "/Library/Preferences/Xhooks/triggerfiles/make_browser_window_big";
my $kiosk_window_size;

if ( ! defined $DISPLAY_RESOLUTION or $DISPLAY_RESOLUTION eq "" ) {
  $kiosk_window_size = "{0, 22, 1024, 764}";
} else {
  if ( $DISPLAY_RESOLUTION =~ /^(\d+)\s+(\d+)/ ) {
    $kiosk_window_size = "{0, 22, $1, $2}";
  } else {
    $kiosk_window_size = "{0, 22, 1024, 764}";
  }
}
my $mainAppSystemname = "Safari";
my $mainAppFilename = "Safari";

#############################################
# RUN
my $timeSinceNoWindow = 0;

my $current_username = getUsername();
my $die_counter = 0;
while ( $current_username eq "" or $current_username eq "root" ) {
  $current_username = getUsername();
  sleep 2;
  $die_counter++;
  die "Could not get username" if $die_counter > 100;
}

print "current_username: $current_username\n" if $debug;
print "kiosk_window_size: $kiosk_window_size\n" if $debug;

my $download_path = ( get_short_os_version() eq "10.4" ) ? "$userHomeFolder/Desktop" : "$userHomeFolder/Downloads";

my $make_new_window = 0;
my $check_for_idle_trigger_file;
my $make_browser_window_big = 1;

while ( 1 ) {

  # CHECK FOR THINGS THAT STOP NORMAL OPPERATING PROCEDURES...
  if ( ! -e $radmind_finished or -e $run_startup_post_maintenance or -e $logout_trigger_file or -e $wait_for_radmind ) {
    print "Sleeping $timeTillNextCheck seconds because: ( ! -e $radmind_finished or $run_startup_post_maintenance or -e $logout_trigger_file or -e $wait_for_radmind )\n" if $debug;
    sleep $timeTillNextCheck;
    next;
  }

  if ( $make_browser_window_big ) {
  
    if ( ! isThisAppRunning( $mainAppFilename, $current_username ) ) {
      print "$mainAppFilename was suppose to be running, but isnt...\n" if $debug;
      sleep $timeTillNextCheck;
      next;
    }
  
    doAppleEvent( "tell application \"$mainAppFilename\" to set bounds of window 1 to $kiosk_window_size" );
    doAppleEvent( "tell application \"$mainAppFilename\" to activate" );
    $theResult = `osascript -e 'try' -e 'tell application "System Events" to set x to (name of processes) contains "ScreenSaverEngine"' -e 'if x then tell application "ScreenSaverEngine" to activate' -e 'end try'`;
    $make_browser_window_big = 0;
  } 

  ######################################
  ## NORMAL OPERATING PROCEEDURES FOLLOW
  ######################################

  if ( ! isThisAppRunning( "ScreenSaverEngine", $current_username ) ) {
    print "------------------------------------------------------------------------\nnot idle\n" if $debug;
    if ( ! isThisAppRunning( $mainAppFilename, $current_username ) ) {
      print "$mainAppFilename was suppose to be running, but isnt...\n" if $debug;
      sleep $timeTillNextCheck;
      next;
    }
  
    if ( $mainAppSystemname eq "Safari" ) {

      # This is above the check because then it is delayed by one while loop iteration
      if ( $make_new_window ) {
        doAppleEvent( "tell application \"$mainAppSystemname\" to make new document" );
        doAppleEvent( "tell application \"$mainAppFilename\" to set bounds of window 1 to $kiosk_window_size" );
        doAppleEvent( "tell application \"$mainAppFilename\" to activate" );
        $make_new_window = 0;
      }

      my $result = doAppleEvent( "tell application \"$mainAppSystemname\" to count windows" );
      if ( $result == 0 ) {
        $make_new_window = 1;
      } elsif ( $result == 1 ) {
        $url_of_window = doAppleEvent( "tell application \"$mainAppSystemname\" to set x to URL of document 1" );
        if ( $url_of_window eq "" ) { # This is a hack because sometimes Safari says it has a window open but it doesn't and the 
          $make_new_window = 1;
        }
      }
    }
  
    # Open Citrix file if present...
    if ( true_var( $KIOSK_OPEN_CITRIX_ICA_FILES ) ) {
      my @number_of_files = `/bin/ls -1 \"$download_path/\"*\".ica\" 2> /dev/null`;
      foreach $file ( @number_of_files ) {
        print "open citrix file $file\n" if $debug;
        chomp $file;
        system "open", $file;
        sleep 5;
        unlink $file;
      }
    }
    
    $check_for_idle_trigger_file = 1;

  } else {
    print "Kiosk is idle\n" if $debug;
    if ( $check_for_idle_trigger_file and -e $make_browser_window_big_trigger ) {
      $make_browser_window_big = 1;
      $check_for_idle_trigger_file = 0;
      unlink $make_browser_window_big_trigger;
    }
  }

  sleep $timeTillNextCheck;
}

################################################################################
sub doAppleEvent {
  my ( $command ) = @_;
  print "osascript -e 'try' -e \'$command\' -e 'end try'\n" if $debug;
  $theResult = `osascript -e 'try' -e \'$command\' -e 'end try'`;
  print $theResult if $debug;
  return $theResult;
}
