#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

# How do you tell if there are any scheduled processes on a box.... 
# (If you aren't using cron)
#
# sudo launchctl list
#
# if in the list you have "edu.utah.scl.nightly.plist", then inspect the file:
#
# /Library/LaunchDaemons/edu.utah.scl.nightly.plist
#
# To see what the schedule is.
#
#	<key>ProgramArguments</key>
#	<array>
#		<string>/Library/Xhooks/Modules/edu.utah.scl.nightly/nightly.pl</string>
#		<string>MH</string>
#		<string>TWFSU</string>
#	</array>
#
# The first string after "/Library/Xhooks/Modules/edu.utah.scl.nightly/nightly.pl" is the days the machine
# will reboot and run radmind (in this case, Monday and Thursday).  The second string
# lists only the days that the machine will reboot.  To configure a machine with these
# settings, edit "/Library/Preferences/Xhooks/edu.utah.scl.nightly/radmind_reboot".  That file will tell you
# the format.
#
# To stop the schedule, type:
#
# sudo launchctl unload /Library/LaunchDaemons/edu.utah.scl.nightly.plist
#
# To restart it, type:
#
# sudo launchctl load /Library/LaunchDaemons/edu.utah.scl.nightly.plist

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
require "xhooks/lib/create_time_interval_launchd.pl";

# This script should create a radmind and reboot launchdaemon plist.

$nightly_script = "/Library/Xhooks/Modules/edu.utah.scl.nightly/nightly.pl";

if ( ! defined $NIGHTLY_RADMIND_REBOOT or $NIGHTLY_RADMIND_REBOOT eq "" ) {
  exit 0;
}

my @args = split ( ' ', $NIGHTLY_RADMIND_REBOOT );

if ( $#args == 3 ) { # 4 args
  create_time_interval_launchd("edu.utah.scl.nightly", $args[0], $args[1], "", "", "", $nightly_script, $args[2], $args[3]);

} elsif ( $#args == 5 ) { # 6 args
  create_time_interval_launchd("edu.utah.scl.nightly", $args[0], $args[1], "", "", "", $nightly_script, $args[2], "-");
  create_time_interval_launchd("edu.utah.scl.nightly_reboot", $args[3], $args[4], "", "", "", $nightly_script, "-", $args[5]);

} else {
  entman_logger ( "log drawer", "Error: Wrong number of args! @args" );    
}

entman_logger ( "log drawer", "Done." );

exit 0;

