#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

my $nightly_maintenance_disabled = "/tmp/down_with_nightly_maintenance";
my $run_radmind_tonight = "/Library/Preferences/Xhooks/triggerfiles/run_maintenance_night";

my $run_maintenance_startup_trigger = "/Library/Preferences/Xhooks/triggerfiles/run_maintenance";

################################################################################
#
# This script runs every night at a specific time specified by launchd.  It's
# purpose is to just reboot or reboot and run mainetenance at startup.
#
# INSTRUCTIONS:
#
# This script is executed by launchd.  It requires a parameter that specifies
# which days it should run radmind $ARGV[0] and which should reboot $ARGV[1].
#
################################################################################

# Check to see if nightly maintenance disable is allowed and set
if ( defined $STAFF and $STAFF eq "True" and -e $nightly_maintenance_disabled ) {
  entman_logger ( "log", "Done. Nightly maintenance and reboots are disabled on this computer." );
  system "\"$remote_logger_script\" RMErr \"Nightly maintenance and reboots have been disabled on this computer.\"";
  exit 0;
}

# Check to see if I should run radmind or reboot
my @thetime = localtime( time );
my $today = $thetime[6];
my $radmind_day = $ARGV[0];
my $reboot_day = $ARGV[1];

if ( -e $run_radmind_tonight ) {

  run_maintenance_sub();

} else {

  if ( defined $radmind_day and $radmind_day ne "" ) {
    run_maintenance_sub() if ( $today == 1 and grep /M/, $radmind_day ); # monday
    run_maintenance_sub() if ( $today == 2 and grep /T/, $radmind_day ); # tuesday
    run_maintenance_sub() if ( $today == 3 and grep /W/, $radmind_day ); # wednesday
    run_maintenance_sub() if ( $today == 4 and grep /H/, $radmind_day ); # thursday
    run_maintenance_sub() if ( $today == 5 and grep /F/, $radmind_day ); # friday
    run_maintenance_sub() if ( $today == 6 and grep /S/, $radmind_day ); # saturday
    run_maintenance_sub() if ( $today == 0 and grep /U/, $radmind_day ); # sunday
  } else {
    entman_logger ( "log drawer", "No radmind parameters were set for this script!" );
  }
  
  if ( defined $reboot_day and $reboot_day ne "" ) {
    reboot() if ( $today == 1 and grep /M/, $reboot_day ); # monday
    reboot() if ( $today == 2 and grep /T/, $reboot_day ); # tuesday
    reboot() if ( $today == 3 and grep /W/, $reboot_day ); # wednesday
    reboot() if ( $today == 4 and grep /H/, $reboot_day ); # thursday
    reboot() if ( $today == 5 and grep /F/, $reboot_day ); # friday
    reboot() if ( $today == 6 and grep /S/, $reboot_day ); # saturday
    reboot() if ( $today == 0 and grep /U/, $reboot_day ); # sunday
  
  } else {
    entman_logger ( "log drawer", "No reboot parameters were set for this script!" );
  }

}

entman_logger ( "log", "Didn't reboot or run radmind because no time was specified." );

exit 0;

sub run_maintenance_sub {
  if ( defined $RADMIND_RUNS_ONLY_IF_LOGGED_OUT and $RADMIND_RUNS_ONLY_IF_LOGGED_OUT eq "True" and is_machine_logged_in() ) {
    entman_logger ( "log", "Didn't run radmind because computer was logged in." );
    # show a message saying radmind was not run because they were logged in
  } else {
    entman_logger ( "log", "Rebooting then will run radmind at startup." );
    system "touch $run_maintenance_startup_trigger";
    if ( $#RADMIND_SERVERS > 0 ) {
      system "touch $use_shadow_server";
    }
    reboot();
  }
}

sub reboot {
  if ( defined $DONT_FORCE_LOGOUT and $DONT_FORCE_LOGOUT eq "True" and is_machine_logged_in() ) {
    entman_logger ( "log", "Didn't reboot because computer was logged in." );    
  } else {
    warn_users_of_restart();
    # Stop Xgrid
    system "killall -9 xgridagentd";
    sleep 5;
    entman_logger ( "log", "Rebooting." );
    unlink $reboot_required;
    system "/sbin/reboot";
  }
}
