#!/usr/bin/perl

# reset_printing_system.pl
#
# This script is a fork of the printerSetup.pl script.  The original header is below
# and changes are indicated there as well.
#
# Runs via the login hook for a user login to MacOS X
#
# Author: Michael Thole
# Version: 0.3 (5/20/03)
# 
# Tasks:
# - Determine machine's ip address
#   - Use it to figure out which printers we should add
#   - We also add several universal printers
# - Cancel existing print jobs
# - Delete exisitng printers
# - Add the printers we decided apon
#   - Set the default printer
#     - default printer is the 1st printer added to the $printers string
#
# 11/06/2008 - James Reynolds
# - Got rid of the IP check completely and it now reads the xhooks variables instead.
#
# 7/11/2008 - James Reynolds
# - Converted this script to use the Xhooks get ip method, because the other method
#   was grabbing the ip of unused interfaces (airport, etc)
#
# 4/10/2008 - James Reynolds
# - Added OS path check for the mappings and definitions.
#
# Version: 0.4 5/26/2004 - Tom Johnson (palantir)
# - gave full path for lpadmin so that other users with different environment variables could use the script
#
# Version 0.4 10/11/2004 - Tom Johnson (palantir)
# - the name of the script is now printerSetup.replaceme - the web server it
#   is on does not like it when the name ends in .pl  You will need to 
#   rename the file so it ends in .pl before it will work properly.  
#
# http://web.ics.purdue.edu/~tjohnson/osxprinterscripts/

use lib "/Library/Xhooks/Modules";
require "xhooks/lib/config.pl";

$osnum_sht = get_short_os_version();
$debug = 0;

$basepath = "/Library/Preferences/Xhooks/edu.utah.scl.printers/$osnum_sht";

$printerDefinitionsPath="$basepath/printerDefinitions.txt";
%printersToAdd = ();
%addPrinterCommands = ();
$defaultPrinter;

### Subroutine declarations ####

sub trim($) {
    $_=($_[0]);
    s/^\s*|\t*|\n//;
    return $_;
}

sub addPrinters(@) {
    foreach $printer (@_) {
    $p=trim($printer);
        if ( $p ne '' ) {
            $printersToAdd{($p)}=true;
            if(!$defaultPrinter) {
                $defaultPrinter = $p;
            }
        }
    }
}

### Procedural code begins ####

open(DATA, $printerDefinitionsPath) || die("Could not open printer definitions!");
@defData=<DATA>;
close(DATA);

# make our hash table of printer and their defining commmands
foreach $line (@defData) {
    chop($line);
    ($line)=split(/\#/,$line);
    ($name,$command)=split(/ |\t/,$line, 2);
    $command=trim($command);
    $addPrinterCommands{($name)}=$command;
}

if ( nonempty_array ( @PRINTERS ) ) {
	addPrinters(@PRINTERS);
}

# cancel current tasks
print "canceling all printer tasks...\n" if $debug;;
$result=`cancel -a` if ! $debug;

if ( ! true_var ( $DO_NOT_REMOVE_PRINTERS ) ) {
	# remove exiting printers
	@curPrinterList=split(/\n/,`lpstat -p | grep printer | awk '{ print \$2 }'`);
	foreach $p (@curPrinterList) {
		print "removing $p\n" if $debug;
		$result=`/usr/sbin/lpadmin -x $p` if ! $debug;
	}
}

# create and enable printers from our %printersToAdd hash
foreach my $key ( keys %printersToAdd ) {
    my $value = $printersToAdd{$key};
    if($value == true) {
        $command=$addPrinterCommands{$key};
        if($command) {
            if ( $debug ) {
                 print "print: $key\n";
                 print "$command\n" if $debug;
                 if ( $osnum_sht eq "10.5" ) {
                     print  "/usr/sbin/accept $key\n";
                     print "/usr/sbin/cupsenable $key\n";
                 }
            } else {
                 `$command`;
                 if ( $osnum_sht eq "10.5" ) {
                     system "/usr/sbin/accept", $key;
                     system "/usr/sbin/cupsenable", $key;
                 }
            }
        }
        else {
            print "Error: No printer command defined for '$key'\n";
        }
    }
}

# set the default printer, depending on which lab...
if($defaultPrinter) {
    print("default printer: $defaultPrinter\n") if $debug;
    `/usr/sbin/lpadmin -d $defaultPrinter`;
}
else {
    print("Error: No default printer defined");
}
