#!/usr/bin/perl

################################################################################
#
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

my $launch_maintenance_script = "/Library/Xhooks/Modules/edu.utah.scl.radmind/launch_maintenance";
my $wait_for_radmind = "/var/log/wait_for_radmind";
my $stopScreensaverTrigger = "/var/log/edu.utah.scl.logoutsaver.stop";
my $run_maintenance_startup_trigger = "/Library/Preferences/Xhooks/triggerfiles/run_maintenance";
my $radmind_finished = "/Library/Preferences/Xhooks/triggerfiles/radmind_finished";
my $loginwindow_launchd = "/Library/Xhooks/Modules/xhooks/resources/com.apple.loginwindow.plist";

################################################################################
my $debug = ( ( defined $ENV{'xhooks_verbose'} and $ENV{'xhooks_verbose'} ) or defined $ARGV[0] ) ? 1 : 0;

print "$0 - started.\n" if $debug;

################################################################################
# Check to see if maintenance needs to run

$runRM = 0;

if ( -e $run_maintenance_startup_trigger ) {
  # It is time to run maintenance!
  print "$run_maintenance_startup_trigger exists, run radmind.\n" if $debug;
  $runRM = 1;
}

if ( ! -e $radmind_finished ) {
  # The Mac was force restarted while Radmind was running, run it again! 
  print "$radmind_finished did not exist, run radmind.\n" if $debug;
  $runRM = 1;
}

my $osnum = `sw_vers -productVersion`;
$osnum =~ s/(\d+\.\d+).*\n/$1/;

if ( $runRM == 1 ) {
    if ( $osnum eq "10.5" ) {

		system "mv /etc/kcpassword /etc/kcpassword_disabled" if -e "/etc/kcpassword";

		if ( ! -e "/Library/LaunchAgents/edu.umich.iHook.loginwindow.plist" ) {
			die "Can not load /Library/LaunchAgents/edu.umich.iHook.loginwindow.plist because it is missing.\n";
		}

		if ( `launchctl list | grep com.apple.loginwindow` eq "" ) {
			print "loginwindow not running, loading $loginwindow_launchd\n" if $debug;
			print "launchctl load $loginwindow_launchd\n" if $debug;
			system 'launchctl', 'load', $loginwindow_launchd;
			while ( ! isThisAppRunning( "loginwindow", "root" ) ) {
				sleep 1;
			}
		}

		while ( `launchctl list | grep edu.umich.iHook.loginwindow` eq "" ) {
			print "`launchctl list` does not contain edu.umich.iHook.loginwindow... waiting...\n" if $debug;
			sleep 1;
		}

		if ( system "launchctl", "start", "edu.umich.iHook.loginwindow" ) {
			die "Launchd could not load edu.umich.iHook.loginwindow: $!";
		}

		while ( 1 ) {
			sleep 120;
			if ( ! isThisAppRunning( $pathToiHook, "root" ) ) {
			  die "iHook never started!!!";
			}
			sleep 1000000;
		}

	} else {
		print "running radmind the 10.4 way.\n" if $debug;
		sleep 15;
		print "\"$launch_maintenance_script\"\n" if $debug;
		system "\"$launch_maintenance_script\"";
		print "/sbin/reboot\n" if $debug;
		system "/sbin/reboot";
	}

} else {
  unlink $wait_for_radmind if -e $wait_for_radmind;
  unlink $stopScreensaverTrigger if -e $stopScreensaverTrigger;
}

exit 0;
