#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use POSIX qw(strftime);
use Time::Local 'timelocal_nocheck';
use Foundation;

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
require "xhooks/lib/perlplist.pl";

my $status_create_entman_prefs = "/Library/Xhooks/Modules/edu.utah.scl.status/create_entmanmenu_prefs.pl";
my $ard_status = "/Library/Xhooks/Modules/edu.utah.scl.status/ard_status.pl";
my $loginpanel_message = "/Library/Preferences/Xhooks/triggerfiles/loginpanel_message";
my $nightly_launchd_plist = "/Library/LaunchDaemons/edu.utah.scl.nightly.plist";

################################################################################
# entman - modules
#
# This script find the status of stuff and runs the scripts that create the
# entman menu prefs and hack the loginwindow.
#
################################################################################

# This is the number of days that must pass before the icon switches from green to red. 
my $days_before_error = 8;

my $error = 0;
my $blankspace = "";
my $errorText=" ";
my $days;
my $day;
my $month;
my $hour;
my $min;
my $sec;

my $next_run;
my $last_run;

unlink $error_message_file if -e $error_message_file;

################################################################################
# Check out the radmind_lastrun file and get all info from it that is needed (for error and last run date).
if ( -e $maintenance_lastrun ) {
  # get the last time it ran
  my @stats = stat ( $maintenance_lastrun );
  my @last_radmind = localtime ( $stats[9] );
  $month = $last_radmind[4] + 1;
  $day = $last_radmind[3];
  $hour = $last_radmind[2];
  $min = $last_radmind[1];
  $sec = $last_radmind[0];

  if ( $month < 10 ) {
    $blankspace .= " ";
  }
  if ( $day < 10 ) {
    $blankspace .= " ";
  }
  if ( $min < 10 ) {
    $min = "0" . $min;
  }
  if ( $sec < 10 ) {
    $sec = "0" . $sec;
  }

  # Last ran for Entman Menu  
  $last_run = strftime "%a, %b %e, %Y @ %l:%M:%S %p ", @last_radmind; # Mon, Jan.  1, 2001 @  4:00:00 PM
  $last_run =~ s'  ' 'g;
  
  # find out if it is too old.
  unless ( -M $maintenance_lastrun < $days_before_error ) {
    entman_logger ( "log", "Found a problem: file \"$maintenance_lastrun\" is too old (meaning radmind hasn't run recently)." );
    system "/bin/echo \"Maintenance hasn't run recently\" >> \"$error_message_file\"";
    $error = 1;
  }
} else {
  entman_logger ( "log", "Found a problem: file \"$maintenance_lastrun\" does not exist." );
  system "/bin/echo \"Maintenance log does not exist\" >> \"$error_message_file\"";
  $error = 1;
  $day="0";
  $month="0";
  $blankspace="  ";
  $last_run = "Last ran file missing";
}

################################################################################
# get the next time radmind is running from the launchd plist

if ( -e $nightly_launchd_plist ) {

  my $dict = loadDefaults( $nightly_launchd_plist );
  my $date = getPlistObject( $dict, "ProgramArguments", 1 )->description->cString();

  if ( defined $date ) {

    # Find the days that radmind runs
    my @days_running = ();
    if ( grep /M/, $date ) {
      push @days_running, 1;
    }
    if ( grep /T/, $date ) {
      push @days_running, 2;
    }
    if ( grep /W/, $date ) {
      push @days_running, 3;
    }
    if ( grep /H/, $date ) {
      push @days_running, 4;
    }
    if ( grep /F/, $date ) {
      push @days_running, 5;
    }
    if ( grep /S/, $date ) {
      push @days_running, 6;
    }
    if ( grep /U/, $date ) {
      push @days_running, 0;
    }
    
    # Get a text representation of it.
    if ( $#days_running + 1 == 0 ) {

      $days = "--";
      $next_run = "No schedule for the next run";
      entman_logger ( "log", "No schedule for the next nightly maintenance." );

    } else {

      # Short
      $days = "";
      foreach $day ( @days_running ) {
        $days .= num_to_day ( $day );
      }
      if ( $#days_running + 1 == 1 ) {
        $days .= ".";
      } elsif ( $#days_running + 1 > 2 ) {
        $days = substr ( $days, 0, 1);
        $days = $#days_running + 1 . $days;
      }

      # Long
      my $next_hour = getPlistObject( $dict, "StartCalendarInterval", "Hour" )->description->cString;
      my $next_min = getPlistObject( $dict, "StartCalendarInterval", "Minute" )->description->cString;
      my $next_date = 0;
      print "next_hour $next_hour next_min $next_min\n";
      foreach $day ( @days_running ) {
        my $date1 = timelocal_nocheck 0, $next_min, $next_hour, $now[3] + (($day-$now[6]+7) % 7) ,$now[4], $now[5];

        $date1 += $date1 > time ? 0 : 604800;
        $next_date = $next_date == 0 ? $date1 : $next_date;
        $next_date = $date1 < $next_date ? $date1 : $next_date;

      }

      $next_run = strftime "%a, %b %e, %Y @ %l:%M:%S %p ", localtime ( $next_date );
      print "next_run $next_run\n";

    }
  } else {

    system "/bin/echo \"$nightly_launchd_plist incorrect\" >> \"$error_message_file\"";
    entman_logger ( "log", "Found a problem: file \"$nightly_launchd_plist\" incorrect." );
    $days = "--";
    $next_run = "Schedule broke";
    $error = 1;

  }
} else {

  system "/bin/echo \"$nightly_launchd_plist missing\" >> \"$error_message_file\"";
  entman_logger ( "log", "Found a problem: file \"$nightly_launchd_plist\" missing." );
  $days = "--";
  $next_run = "Schedule missing";
  $error = 1;

}

################################################################################
# Check for errors

# Radmind
if ( -e $radmind_error_file ) {

  entman_logger ( "log", "Found a problem: radmind error." );

  my $rmerr_text = `cat \"$radmind_error_file\"`;
  
  open ( FILEHANDLE, ">> $error_message_file" );
  print FILEHANDLE "Radmind error\n";
  print FILEHANDLE "$rmerr_text";
  close ( FILEHANDLE );

  entman_logger ( "log", "$rmerr_text" );

  system "\"$remote_logger_script\" RMErr \"I have a radmind error: $rmerr_text\"";

  if ( defined $RADMIND_ERROR_EMAIL and $RADMIND_ERROR_EMAIL ne "" ) {
    system "/usr/local/bin/file_emailer.pl", "-t", $RADMIND_ERROR_EMAIL, "-f", $0, "-p", $radmind_error_file, "-s", "Radmind Error";
  }

  $error = 1;
  $errorText="!";

}

# SMART Status
my $smart_error = "";
foreach my $x ( 0..10 ) {
  if ( -e "/dev/disk$x" ) {
    my $disk = "disk$x";
    my @smart_status = `/usr/sbin/diskutil info $disk | /usr/bin/grep SMART`;
    if ( $smart_status ne "" ) {
      entman_logger ( "log", "$disk $smart_status" );
      if ( ! grep /Verified/, $smart_status and ! grep /Not Supported/, $smart_status ) {
        $smart_error .= "$disk $smart_status "
      }
    } else {
      #print "$disk SMART not supported\n";
    }
  }
}
if ( $smart_error ne "" ) {
  entman_logger ( "log", "Found a problem: SMART error." );
  system "\"$remote_logger_script\" SMART \"SMART Status: $smart_error\"";
  system "/bin/echo \"SMART error\" >> \"$error_message_file\"";
  $error = 1;
  $errorText="@";
}

################################################################################
# Finish the title.
my $menuTitle;
my $loginTitle;
my $ardTitle;
if ( $errorText eq " " ) {
  $menuTitle="$days$month.$day";
  $loginTitle="$days$month.$day $blankspace";
  $ardTitle="$days$month.$day\@$hour:$min:$sec";
} else {
  $menuTitle="$errorText$days$month.$day";
  $loginTitle="$errorText$days$month.$day$blankspace";
  $ardTitle="$errorText$days$month.$day\@$hour:$min:$sec";
}

################################################################################
################################################################################
################################################################################

################################################################################
# Write the EntmanMenu pref file
system "\"$status_create_entman_prefs\" \"$menuTitle\" $error \"$next_run\" \"$last_run\" &";

################################################################################
# Write the ARD pref file
system "\"$ard_status\" \"$ardTitle\" &";

################################################################################
# Hack the loginpanel
system "echo $loginTitle $error > $loginpanel_message";

exit 0;

sub num_to_day {
  my ( $day ) = @_;
  if ( $day == 1 ) {
    return "M";
  } elsif ( $day == 2 ) {
    return  "T";
  } elsif ( $day == 3 ) {
    return  "W";
  } elsif ( $day == 4 ) {
    return  "H";
  } elsif ( $day == 5 ) {
    return  "F";
  } elsif ( $day == 6 ) {
    return  "S";
  } elsif ( $day == 7 ) {
    return  "U";
  }
  return "?";
}
