#!/usr/bin/perl

################################################################################
# Copyright (c) 2005 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

################################################################################
# entman - modules
#
# Create Entman Menu prefs (the Entman Menu is in the menu bar!)
#
################################################################################

use Foundation;
require "xhooks/lib/perlplist.pl";

my $menu_title = $ARGV[0];
my $error = $ARGV[1];
my $next_run = $ARGV[2];
my $last_run = $ARGV[3];

my $error_message;

if ( $error ) {
  if ( -e $error_message_file ) {
    $error_message = `/bin/cat \"$error_message_file\"`
  } else {
    $error_message = "There was an error";
  }
} else {
  $error_message = "";
}

create_entman_menu_pref($menu_title, $next_run, $last_run, $error_message);

entman_logger ( "log", "Done." );

exit 0;

################################################################################

sub justify {
  my ( $text, $max_length ) = ( @_ );

  my @paragraphs = split /\n/, $text;

  my @lines = ();
  
  foreach my $paragraph ( @paragraphs ) {

    my $current_line = "";
    my @words = split / /, $paragraph;
    foreach my $word ( @words ) {
      $current_line .= "$word ";
      my @letters = split (//, $current_line);
      if ( $#letters > $max_length ) {
        chomp $current_line;

        push @lines, $current_line;

        $current_line = "";
      }
    }
    if ( $current_line ne "" ) {
      push @lines, $current_line;
    }

    push @lines, "";

  }
  return @lines;
}

sub create_entman_menu_pref {
  my ( $menu_title, $next_run, $last_run, $error_message ) = ( @_ );

  my $PREF_TITLE_NAME= "title";
  my $PREF_IMAGE_NAME = "status";
  my $PREF_IMAGE_NORMAL = "normal";
  my $PREF_IMAGE_ERROR = "error";
  my $PREF_MENU_IMAGE_PATH = "menuImagePath";
  my $PREF_MENU_ARRAY = "menuArray";
  my $PREF_MENU_ITEM_TEXT = "menuTitle";
  my $PREF_MENU_ACTION = "menuAction";
  my $PREF_MENU_INDENT = "menuIndentationLevel";
  my $MENU_SEPARATOR = "separator";
  my $ACTION_LOGOUT = "logout";
  my $ACTION_RESET = "reset";
  my $ACTION_EDIT_PREFS = "editMyPrefs";
  my $ACTION_QUIT = "quit";

  my @menu = (
            { "$PREF_MENU_ITEM_TEXT"=>"Last Run",	"$PREF_MENU_ACTION"=>"",	"$PREF_MENU_INDENT"=>"" },
            { "$PREF_MENU_ITEM_TEXT"=>"$last_run",	"$PREF_MENU_ACTION"=>"",	"$PREF_MENU_INDENT"=>"1" },
            { "$PREF_MENU_ITEM_TEXT"=>"Next Run",	"$PREF_MENU_ACTION"=>"",	"$PREF_MENU_INDENT"=>"" },
            { "$PREF_MENU_ITEM_TEXT"=>"$next_run",	"$PREF_MENU_ACTION"=>"",	"$PREF_MENU_INDENT"=>"1" },
          );

  my $image;
  
  if ( $error_message ) {

    my @errors = ( { "$PREF_MENU_ITEM_TEXT"=>"Errors",	"$PREF_MENU_ACTION"=>"",	"$PREF_MENU_INDENT"=>"0" } );

    my @message_lines = justify( $error_message, 45 );

    foreach $line ( @message_lines ) {
      push ( @errors, { "$PREF_MENU_ITEM_TEXT"=>"$line", "$PREF_MENU_ACTION"=>"", "$PREF_MENU_INDENT"=>"1" } );
    }

    push ( @menu, @errors );

    $image = $PREF_IMAGE_ERROR;

  } else {

    $image = $PREF_IMAGE_NORMAL;

  }

  my $image_path = ""; # leave this blank if you declare the $image variable.

  my %pref = (
               "$PREF_TITLE_NAME"		=>	"$menu_title",
               "$PREF_IMAGE_NAME"		=>	"$image",
               "$PREF_MENU_IMAGE_PATH"	=>	"$image_path",
               "$PREF_MENU_ARRAY"		=>	\@menu
             );

  my $dict = cocoaDictFromPerlHash( %pref );

  #printObject( $dict );

  if ( ! saveDefaults( $dict, "/Library/Preferences/edu.utah.scl.entmanmenu.plist" ) ) {
    entman_logger ( "log", "Could not write to file /Library/Preferences/edu.utah.scl.entmanmenu.plist" );
  }

}
