#!/bin/sh

# iHook script on steroids
#
# runs everything in ${HOOKSDIR} beginning with ${hook_subscript}
#
# This script expects these environment variables to be defined (and exported):
#   hook_subscript - Required
#   ihookbackground - Optional
#

export PATH=/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin:/usr/local/sbin

#################################################################################
# Remove the following if you want to run this script independent of Xhooks
# (see the script requirements and make sure you supply them manually)
##

entman_config="/Library/Xhooks/Modules/xhooks/lib/config.sh"
entman_lib="/Library/Xhooks/Modules/xhooks/lib/common.sh"

if [ ! -f "${entman_config}" ]; then
	logger -s -t \"edu.utah.scl.xhooks-`basename $0`\" -p user.info ${entman_config} is missing. Exiting...
	exit 1
fi

. "${entman_config}"
. "${entman_lib}"

#################################################################################
# This scripts' requirements
# (supply these manually if you run this script out of the Xhooks context)
##

# vars this script requires
#
HOOKSDIR="/Library/Xhooks/Modules/xhooks/hooks"

# subroutines this script requires:
#
my_logger() { entman_logger "$@"; }

#################################################################################
#################################################################################

# Save the PID
if [ "${1}" = "-pidfile" ]; then
	pidfile=$2
	[ -e "$pidfile" ] && rm -rf $pidfile
	echo $$ > $pidfile
	shift 2
fi

# Save stdout
exec 3>&1

# Check for hooksdir
if [ ! -d ${HOOKSDIR} ]; then
	my_logger "log drawer" "${HOOKSDIR} MISSING!"
	exit 1
fi

if [ "$hook_subscript" = "" ]; then
	my_logger "log drawer" "The variable hook_subscript is not defined!"
	exit 1
fi

# become front application, disable UI and show barberpole
if [ -e "${ihookbackground}" ]; then
	echo %BACKGROUND ${ihookbackground}
	echo %WINDOWLEVEL HIGH
	echo %BECOMEKEY
	echo %UIMODE AUTOCRATIC
	echo %BEGINPOLE
fi

#################################################################################

# Run (F)irst scripts
for hook in ${HOOKSDIR}/${hook_subscript}E*; do
	if [ -e "$hook" -o -L "$hook" ]; then
		runhook $hook false "$@"
	fi
done
# Run (D)etached scripts
for hook in ${HOOKSDIR}/${hook_subscript}D*; do
	if [ -e "$hook" -o -L "$hook" ]; then
		runhook $hook true "$@"
	fi
done
wait
# Run (L)ast scripts
for hook in ${HOOKSDIR}/${hook_subscript}L*; do
	if [ -e "$hook" -o -L "$hook" ]; then
		runhook $hook false "$@"
	fi
done

if [ "${1}" != "" ]; then
	rm -f $pidfile
fi

#################################################################################

echo %ENDPOLE

#################################################################################

exit 0
