my $network_test_script = "/Library/Xhooks/Modules/xhooks/bin/networktest.pl";
my $trash_can = "/Library/Preferences/Xhooks/Caches/Trash";
my $maintenanceNotification = "/Library/Xhooks/Utilities/Helpers/Maintenance Notification.app";
my $network_is_up = "/Library/Preferences/Xhooks/triggerfiles/networkisup";

sub true_var {
  my ( $XHOOKS_VAR ) = @_;
  if ( defined $XHOOKS_VAR and $XHOOKS_VAR eq 'True' ) {
    return 1;
  } else {
    return 0;
  }
}

sub nonempty_var {
  my ( $XHOOKS_VAR ) = @_;
  if ( defined $XHOOKS_VAR and $XHOOKS_VAR ne '' ) {
    return 1;
  } else {
    return 0;
  }
}

sub nonempty_array {
  if ( defined @_ and $#_ >= 0 ) {
    return 1;
  } else {
    return 0;
  }
}

sub split_string_with_quotes {
  my ( $code ) = @_;
  
  # This will parse a string into an array of strings split by spaces or tabs.
  # It stops when it reaches // or a new line
  # It saves spaces or tabs if they are in quoted strings
  # It saves quotes inside of quoted strings or if escaped (I think)
  # It does not save spaces if escaped outside of quotes

  my $debug_parse = 0; # print debug info
  my $debug_pushed = 0; # print debug info

  my @file_chars = split //, $code;
  my $char;

  my $escape_state = 0;
  my $acquire_not_idle_state = 0; # 1 is acquire, 0 is idle
  my $quote_state = 0; # 0 is off, 1 is " , 2 is '
  my $comment_state = 0; # 0 is off, 1 is // , 2 is /*
  my @tokens = ();
  my $current_word = "";
  my $line_number = 1;
  my $char_number = 0;
 
  my %white_char = (
    ' '=>1,
    "\t"=>1,
  );

  print "Scanning array into tokens\n" if $debug_parse;

  my $counter = -1;
  while ( $#file_chars > $counter ) {

    $char = get_next_token( \$counter, \@file_chars );

    if ( ! $quote_state and ( $char eq "\r" or $char eq "\n" ) ) {
      print " - end of line found." if $debug_parse;
      last;
    }

    # handle */
    if ( ! $quote_state and $char eq '/' ) {
      if ( $comment_state == 2 and $counter > 0 and $file_chars[$counter-1] eq '*' ) {
        $comment_state = 0;
        print " - blockcomment off: $current_word\n" if $debug_parse;
        next;
      }
    }
    # handle comment state
    if ( $comment_state ) {
      print "$char" if $debug_parse;
      next;
    }
    # handle //
    if ( ! $quote_state and $char eq '/' ) {
      if ( $counter > 0 and $file_chars[$counter-1] eq '/' ) {
        chop $current_word;
        print " - linecomment on: $current_word\n" if $debug_parse;
        last;
      }
    }
    # handle /*
    if ( ! $quote_state and $char eq '*' ) {
      if ( $counter > 0 and $file_chars[$counter-1] eq '/' ) {
        $comment_state = 2;
        chop $current_word;
        print " - blockcomment on: $current_word\n" if $debug_parse;
        next;
      }
    }

    print $char if $debug_parse;

    $char_number++; # This isn't entirely accurate, can be off by one...

    # Handle quotes

    if ( $char eq '"' ) {
      if ( $counter > 0 and ! $escape_state ) {
        if ( $quote_state == 0 ) {
          $quote_state = 1;
          $escape_state = 0;
          print " - q1on\n" if $debug_parse;
          next;
        } elsif ( $quote_state == 1 ) {
          $quote_state = 0;
          print " - q1off\n" if $debug_parse;
          print "pushed1 $current_word\n" if $debug_pushed;
          push @tokens, { 'token'=>$current_word, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'quoted string', 'pushedat'=>'1' };
          $current_word = "";
          next;
        }
      }
    }
    if ( $char eq "'" ) {
      if ( $counter > 0 and ! $escape_state ) {
        if ( $quote_state == 0 ) {
          $quote_state = 2;
          $escape_state = 0;
          print " - q2on\n" if $debug_parse;
          next;
        } elsif ( $quote_state == 2 ) {
          $quote_state = 0;
          print " - q2off\n" if $debug_parse;
          print "pushed2 $current_word\n" if $debug_pushed;
          push @tokens, { 'token'=>$current_word, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'quoted string', 'pushedat'=>'2' };
          $current_word = "";
          next;
        }
      }
    }
    if ( $quote_state ) {
      if ( $escape_state ) {
        $escape_state = 0;
      } elsif ( $char eq "\\" ) {
        $escape_state = 1;
        next;
      }
      $current_word .= $char;
      next;
    }

    # Parse words

    if ( $white_char{$char} ) {
      if ( ! $quote_state ) {
        if ( $counter > 0 and $white_char{$file_chars[$counter-1]} ) {
          next;
        } else {
          $acquire_not_idle_state = 0;
          if ( $current_word ne "" ) {
            print "pushed3 $current_word\n" if $debug_pushed;
            push @tokens, { 'token'=>$current_word, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'non-reserved word', 'pushedat'=>'3' };
          }
          $current_word = "";
          next;
        }
      } else {
        $current_word .= $char;
        next;
      }
    }

    if ( ! $white_char{$char} ) {
      $current_word .= $char;
      $acquire_not_idle_state = 1;
      next;
    } elsif ( $acquire_not_idle_state ) {
      print "pushed4 $current_word\n" if $debug_pushed;
      push @tokens, { 'token'=>$current_word, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'non-reserved word', 'pushedat'=>'4' };
      $current_word = "";
      print "pushed5 $char\n" if $debug_pushed;
      push @tokens, { 'token'=>$char, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'CHAR?', 'pushedat'=>'5' };
      $acquire_not_idle_state = 0;
      next;
    } else {
      print "pushed6 $char\n" if $debug_pushed;
      push @tokens, { 'token'=>$char, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'?', 'pushedat'=>'6' };
      next;
    }

  } # foreach
  if ( $current_word ne "" ) {
    print "pushed7 $current_word\n" if $debug_pushed;
    push @tokens, { 'token'=>$current_word, 'line'=>$line_number, 'char'=>$char_number, 'name'=>'?2', 'pushedat'=>'7' };
  }
  print "\n" if $debug_parse;
  
  # Make sure everything closed.
  if ( $quote_state ) {
    print "Unclosed qoute!\n";
    print Dumper(\@tokens);
    die "ERROR: splitStringWithQuotes -  Unclosed qoute!\n";
  }
  if ( $comment_state ) {
    print "Unclosed comment!\n";
    print Dumper(\@tokens);
    print "ERROR: splitStringWithQuotes - Unclosed comment!\n";
  }
  
  print "Just the tokens please\n" if $debug_parse;
  my @just_tokens = ();
  foreach my $just_token ( @tokens ) {
    print $$just_token{'token'}."\n" if $debug_parse;
    push @just_tokens, $$just_token{'token'};
  }
  
  return @just_tokens;
}

sub get_next_token {
  my ( $counter_pointer, $file_chars_pointer ) = @_;
  #print "trace - get_next_token\n" if $debug_trace;
  $$counter_pointer++;
  return $$file_chars_pointer[$$counter_pointer];
}

sub updateByHost {
	my ( $byHostDir ) = @_;
  		
	# Set test value for first UUID octet (for determining hardware)
	my $testUUID = "00000000";
	
	# Discover value for first UUID octet
	my $partialUUID = `ioreg -rd1 -c IOPlatformExpertDevice | grep -E '(UUID)' | cut -c 27-34`;
	chomp $partialUUID;
	
	# Compare first UUID octet versus test value, if equivalent
	# the machine is PPC or early Intel
	if ($partialUUID eq $testUUID) {
		$UUID = `ioreg -rd1 -c IOPlatformExpertDevice | grep -E '(UUID)' | cut -c 51-62`;
		$UUID = lc($UUID);  
	}
	# Otherwise, the machine is an Intel with full support for UUIDs
	else {
		$UUID = `ioreg -rd1 -c IOPlatformExpertDevice | grep -E '(UUID)' | cut -c 27-62`;
	}
	
	chomp $UUID;	
	
	# Replace the old user ByHost files with appropriately named versions
	opendir BYHOSTDIR, $byHostDir or entman_logger ( "log drawer", "$0 can't open $byHostDir" );
	my @prefFiles = grep /\.plist$/, readdir BYHOSTDIR;
	closedir BYHOSTDIR;
	
	foreach my $file (@prefFiles) 
	{
		my $newFile = $file;
		$newFile =~ s/^(.*)\.[^.]*\.plist$/$1.$UUID.plist/;
		$file = "$byHostDir/$file";
		$newFile = "$byHostDir/$newFile";
		unless (-e $newFile) 
		{
			system( "/bin/mv", $file, $newFile );
		}
	}
}

sub idleTime {
  foreach (`/usr/sbin/ioreg -c IOHIDSystem`) {
    if( /HIDIdleTime" = (\d*)/ ) {
      return $1/1000000000 if $1 ne "";
    }
  }
  return 0; # just in case
}

################################################################################
sub removeAllFilesInFolderExcept {
  my ($pathToFolder, @allowedFiles) = @_;
  if (! -d $pathToFolder ) {
    print "removeAllFilesInFolderExcept $pathToFolder folder doesn't exist\n";
    return;
  }
  @currentFiles = getFileFolderList($pathToFolder);
  @killMe = ();
  $repeat1 = @currentFiles;
  $repeat2 = @allowedFiles;
  for ($i = 0; $i < $repeat1; $i++) {
    $iDie = 1;
    for ($j = 0; $j < $repeat2; $j++) {
      if($allowedFiles[$j] eq $currentFiles[$i]) {
        $iDie = 0;
      }
    }
    if ($iDie) {
      push(@killMe,$currentFiles[$i]);
    }
  }
  $repeat1 = @killMe;
  chdir $pathToFolder;
  for ($i = 0; $i < $repeat1; $i++) {
    print "/usr/bin/chflags -R nouchg \"$killMe[$i]\"\n";
    print "/bin/rm -rf \"$killMe[$i]\"\n";
    system "/usr/bin/chflags -R nouchg \"$killMe[$i]\"";
    system "/bin/rm -rf \"$killMe[$i]\"";
  }
}

################################################################################
sub get_dated_folder_for_path {
  my ( $dated_material_folder ) = @_;
  $dated_material_folder = $dated_material_folder."/" if ( substr $dated_material_folder, -1 ) ne "/";
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime time;
  $mon++;
  $year += 1900;
  my @dated_folder_list = getFileFolderList($dated_material_folder);
  @dated_folder_list = grep( !/^\.DS_Store$/, @dated_folder_list );
  foreach my $ii ( @dated_folder_list ) {
#    print "$ii\n";
    if ( index ( $ii, "-" ) >= 0 ) {
      my @parts = split '-', $ii;
      my @date1 = split '\.', $parts[0];
      my @date2 = split '\.', $parts[1];
      $date1[0] -= 12 if $date1[0] > $date2[0];
      if ( $mon == $date1[0] ) {
        if ( $mon < $date2[0] ) {
          if ( $mday >= $date1[1] ) {
#            print "1a\n";
            return $dated_material_folder.$ii;
          }
        } elsif ( $mon == $date2[0] ) {
          if ( $mday >= $date1[1] and $mday <= $date2[1] ) {
#            print "1b\n";
            return $dated_material_folder.$ii;
          }
        }
      } elsif ( $mon > $date1[0] and $mon < $date2[0] ) {
#          print "2\n";
          return $dated_material_folder.$ii;
      } elsif ( $mon == $date2[0] ) {
        if ( $mday <= $date2[1] ) {
#          print "3\n";
          return $dated_material_folder.$ii;
        }
      }
    } else {
      if ( $ii eq ( sprintf "%02d.%02d", $mon, $mday ) ) {
        return $dated_material_folder.$ii;
      } 
      if ( $ii eq ( sprintf "%02d.%02d.%02d", $mon, $mday, $year ) ) {
        return $dated_material_folder.$ii;
      } 
    }
  }  
}

################################################################################
sub get_dated_folder_for_path2 {
  my ( $dated_material_folder ) = @_;
  
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime time;
  $mon++;
  $year += 1900;
  my $todays_folder = sprintf "$dated_material_folder%02d.%02d", $mon, $mday;
  my $todays_folder2 = sprintf "$dated_material_folder%02d.%02d.%02d", $mon, $mday, $year;
  
  my $found_folder = "";
  $found_folder = $todays_folder if -e $todays_folder;
  $found_folder = $todays_folder2 if -e $todays_folder2;
  return $found_folder;
  
}

################################################################################
sub get_rand_items_in_dated_folder_for_path {
  my ( $dir_to_search, $count ) = @_;
  
  my $dir1 = get_dated_folder_for_path( $dir_to_search );
  my @items = ();
  if ( $dir1 ne "" ) {
    my @dir2 = getFileFolderList($dir1);
    @dir2 = grep(!/^\.DS_Store$/, @dir2);
    for ( 1..$count ) {
      push @items, $dir1."/".$dir2[int (rand()*($#dir2 + 1))];
    }
  }
  return @items;
}

################################################################################
sub get_rand_items_in_folder {
  my ( $dir1, $count ) = @_;
  
  my @items = ();
  if ( $dir1 ne "" ) {
    my @dir2 = getFileFolderList($dir1);
    @dir2 = grep(!/^\.DS_Store$/, @dir2);
    for ( 1..$count ) {
      push @items, $dir1."/".$dir2[int (rand()*($#dir2 + 1))];
    }
  }
  return @items;
}

################################################################################
sub readOneLineFile {
  my ($filename) = @_;
  open (FILEHANDLE, "< $filename");
  $line = <FILEHANDLE>;
  close (FILEHANDLE);
  chomp $line;
  return $line;
}

################################################################################
sub getFileModTime {
  my ($filename) = @_;
  # See page 572 in PERL Back Book for the definition of the following:
  ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($filename);
  return $mtime;
}

################################################################################
sub put_in_trash {
  my ( $my_folder ) = @_;

  my $temp = 1;
  while ( -e "$trash_can/$temp" ) {
    $temp++;
  }
  system "/bin/mv \"$my_folder\" \"$trash_can/$temp\"";
  
}

sub empty_trash {
  system "/usr/bin/chflags -R nouchg \"$trash_can/\"*  2>/dev/null && /bin/rm -rf -- \"$trash_can/\"* &";
}

################################################################################
sub say {
  if ( $debug) {
    my ($message) = @_;
    if ( $debugWithoutVoice != 1) {
      system "/usr/bin/say \"$message\" &"
    }
    print $message . "\n";
  }
}

#########.#########.#########.#########.#########.#########.#########.#########.#
sub findNumberOfFoldersAndFiles {
  my ($folder) = @_;
  if ( -d $folder ) {
    my @dirlist=();
    opendir(DIR, "$folder");
    @dirlist=readdir(DIR);
    closedir(DIR);
    @dirlist = grep(!/^\.\.?$/, @dirlist);
    return $#dirlist+1;
  } else {
    return 0;
  }
}

################################################################################
sub getFileFolderList {
  my ($folder) = @_;
  if ( -d $folder ) {
    my @dirlist=();
    opendir(DIR, "$folder");
    @dirlist=readdir(DIR);
    closedir(DIR);
    @dirlist = grep( !/^\.\.?$/, @dirlist );
    return @dirlist;
  } else {
    return ();
  }
}

sub count_home_folder_cache_folders {
  # Do this little safety thing (just in case anyone goes into this folder with the Finder)...
  my @dirlist = getFileFolderList( $homeFolderBackups );
  @dirlist = grep( !/^\./, @dirlist );
  return $#dirlist+1;
}

################################################################################
sub isThisAppRunning  {
  my ($appToCheck, $username) = @_;
  open (PROC, "/bin/ps -wwxo pid,command -U $username |" );
  @proclist = <PROC>;
  close PROC;
  return grep ( /$appToCheck/i, @proclist);
}

# Other ways to do this, I should probably switch:
#   # my favorite
#   use Mac::Apps::Launch;
#   print IsRunning('com.adobe.photoshop');

#   use Mac::Processes;
#   while (my($psn, $psi) = each %Process) {
#      kill 15, GetProcessPID($psn) if $psi->processName =~ /Photoshop/i;
#   }


################################################################################
sub getPID {
  my ($app, $username) = @_;
  if (defined $username) {
    open (PROC, "/bin/ps -wwxo pid,command -U $username | grep \'$app\' |" );
    @proclist = <PROC>;
    close PROC;
    chomp(@proclist);
    return @proclist;
  } else {
    $id = `ps -awwxo pid,command | grep \"$app\" | grep -v grep | awk \'{ print \$1 }\'`;
    chomp $id;
    return $id;
  }
}

################################################################################
sub getUsername {
  my $username;
  if ( -e $login_name_trigger) {
    $username = readOneLineFile( "$userHomeFolder/$login_name_trigger" );
  } else {
    $username = `/usr/bin/who | /usr/bin/grep console | /usr/bin/awk \'{ print \$1 }\'`;
    chomp $username;
  }
  return $username;
}

################################################################################
# This script renames $path_to_log so that the lowest number is always the
# newest.  The oldest that is greater than $number_of_backups is deleted.
#
sub roll_log {
  my ($path_to_log, $number_of_backups) = @_;
  if (-e $path_to_log) {
    if (-e $path_to_log.".bak$number_of_backups") {
      unlink $path_to_log.".bak$number_of_backups";
    }
    for ($i = $number_of_backups ; $i > 1 ; $i-- ) {
      if (-f $path_to_log.".bak".($i-1)) {
        system "/bin/mv -f \"$path_to_log".".bak".($i-1)."\" \"$path_to_log".".bak$i\"";
      }
    }
    system "/bin/mv -f \"$path_to_log\" \"$path_to_log".".bak1\"";
  }
}

################################################################################
sub timewindow {
  my ($start_time, $end_time) = @_;
  my $hour = (0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23) [(localtime) [2]];
  if ( ( $end_time < $start_time and ( $hour >= $start_time or $hour < $end_time ) ) 
         or
       ( $hour >= $start_time and ( $hour < $end_time ) ) ) {
    return 1;
  }
  return 0;
}

################################################################################
# Create a time stamp from a time value: M.DD.YYYY at HH.MM.SS AM
sub getTimeStamp {
  @_ = (localtime($_[0]))[ 4, 3, 5, 2, 1, 0 ];
  $_[2]+=1900;
  $_[0]++;
  if ($_[3] > 11) {
    $_[3] = $_[3] - 12;
    $theAMPM = "PM";
  } else {
    $theAMPM = "AM";
  }
  $_[3] = ( $_[3] < 1 ) ? 12 : $_[3];
  return sprintf '%d.%02d.%4d at %02d.%02d.%02d %s', @_, $theAMPM;
}

################################################################################
# Create a time stamp from a time value YYYY.MM.DD.HH.MM
sub getTimeStamp2 {
  @_ = ( localtime ($_[0]) ) [ 5, 4, 3, 2, 1 ];
  $_[0]+=1900;
  $_[1]++;
  return sprintf '%4d.%02d.%02d.%02d.%02d', @_;
}
sub getTimeStampOfFile {
  return getTimeStamp2((stat("$_[0]"))[9]);
}
sub compare_ips {
  ################################################
  # compare_ips v1
  ################################################

  my ($ip1, $ip2) = @_;

  my @ip_parts1 = split ('\.', $ip1);
  my @ip_parts2 = split ('\.', $ip2);
  my $match = 1;

  for ($j=0;$j<4;$j++) {
    ############
    ############Debug #print $ip_parts1[$j] . " " . $ip_parts2[$j] . "\n";
    ############
    # Compare each part of the Ip and look for a match.
    if ($ip_parts1[$j] eq '*') {
      # Found *, match!
      last;
    } elsif (index ($ip_parts1[$j], '-') >= 0) {
      # Found -, checking range
      my @ip_range = split ('-', $ip_parts1[$j]);
      $number = @ip_range;
      if ($number == 1) {
        if ($ip_parts2[$j] >= $ip_range[0] ) {
          # IP is greater than range, match!
          last;
        } else {
          # IP is outside of range, no match!
          $match = 0;
          last;
        }          
      } elsif ($number == 2) {
        if ($ip_parts2[$j] >= $ip_range[0] and $ip_parts2[$j] <= $ip_range[1] ) {
          # IP is between range, match!
          last;
        } else {
          # IP is outside of range, no match!
          $match = 0;
          last;
        }
      } else {
        # what the heck?
        $match = 0;
        last;
      }
    } elsif ($ip_parts1[$j] == $ip_parts2[$j]) {
      #keep going
    } else {
      $match = 0;
      last;
    }
  }
  return $match;
}

sub warn_users_of_restart {
  if ( is_machine_logged_in() ) {
    system "touch \"$reboot_required\"";
    system "/usr/bin/open \"$maintenanceNotification\"";
    sleep $restart_warning_minutes_if_user_logged_in * 60;
  }
}

sub is_machine_logged_in {
  my $username = `/usr/bin/who | /usr/bin/grep console | /usr/bin/awk \'{ print \$1 }\'`;
  chomp $username;
  # Check to see if someone is logged in
  if ( $username eq '' ) {
    return 0;
  } else {
    return 1;
  }

}

sub wait_for_network {
  if ( ! -e $network_is_up ) {
    system "$network_test_script";
    system "/usr/bin/touch \"$network_is_up\"";
  }
}

sub get_ip {
  ################################################
  # get_ip v1.e
  # Gotta love that networking doesn't always work..
  # this method really sucks, but I'm not sure what else to do...
  ################################################

  my $my_ip = `/sbin/ifconfig en0 | egrep "inet .* broadcast" | cut -f2 -d' '`;
  chomp $my_ip;
  return $my_ip if $my_ip ne "";

  $my_ip = `/sbin/ifconfig en1 | egrep "inet .* broadcast" | cut -f2 -d' '`;
  chomp $my_ip;
  return $my_ip if $my_ip ne "";

  $my_ip = `/usr/local/bin/get_ip.pl`;
  chomp $my_ip;
  return $my_ip;
}

sub get_ip_old {
  ################################################
  # get_ip v1.c
  ################################################

  my $my_ip = `/sbin/ifconfig en0 | grep "inet " | cut -f2 -d' '`;
  chomp $my_ip;
  if ( $my_ip ne "" ) {
     return $my_ip;
  }
  return "0.0.0.0";
}

sub get_ip_older {
  ################################################
  # get_ip v1.b
  ################################################

  ################################################
  # Try ifconfig
  my $my_ip = `/sbin/ifconfig en0 | grep "inet " | cut -f2 -d' '`;
  chomp $my_ip;
  if ( $my_ip ne "" ) {
     return $my_ip;
  }

  # It didn't work.  That must mean the network is down.
  entman_logger ( "log", "ifconfig en0 returned no ip.  The network must be down." );

  ################################################
  # Try ncutil  
  if ( -e "/usr/sbin/ncutil" ) {
    $my_ip = `/usr/sbin/ncutil -readprop \"/Automatic/Built-in Ethernet/IPv4\" ip-address`;
    chomp $my_ip;
    $my_ip = substr($my_ip, 1);
    if ($my_ip != "") {
       return $my_ip;
    }
  }

  # It didn't work.  That must mean something not good.
  entman_logger ( "log", "ncutil didn't work.  Beats me..." );

  ################################################
  # Ok... Try just reading the network preferences  
  open (FILEHANDLE, "< $networkprefs");
  @lines = <FILEHANDLE>;
  close (FILEHANDLE);
  $depth = 0;
  foreach $i (@lines) {
    if ($depth == 0) {
      if (index ($i, "<key>0</key>") > 0) {
        $depth++;
      } 
    } elsif ($depth == 1) {
      if (index ($i, "<key>IPv4</key>") > 0) {
        $depth++;
      } 
    } elsif ($depth == 2) {
      if (index ($i, "<key>Addresses</key>") > 0) {
        $depth++;
      } 
    } elsif ($depth == 3) {
      if (index ($i, "<array>") > 0) {
        $depth++;
      } 
    } elsif ($depth == 4) {
      if (index ($i, "<string>") > 0) {
        $my_ip = substr($i, index($i, ("<string>"))+8, index($i, ("</string>")) - (index($i, ("<string>"))+8));
        last;
      } 
    }
  }
  if ($my_ip != "") {
     return $my_ip;
  }

  # It didn't work.  That must mean something not good.
  entman_logger ( "log", "Couldn't figure out the ip from the pref file.  Beats me..." );
  return;
}

sub findUpServers {
  my ( $port, @server_list ) = @_;
  my @new_server_list = ();
  foreach $server ( @server_list ) {
    if ( server_reachable( $server, $port ) ) {
      push @new_server_list, $server;
    }
  }
  return @new_server_list;
}

sub server_reachable {
  my ( $server, $port ) = @_;
  my $up = 0;
  $hping2 = "/usr/local/bin/hping2";
  if ( -e $hping2 and defined $port ) {
    #print "$hping2 -I en0 -c 4 -S $server -p $port\n";
    open ( OUTPUT, "$hping2 -I en0 -c 4 -S $server -p $port |" );
    while ( <OUTPUT> ) {
      $up = 1 if ( grep /SA/, $_ );
    }
    close ( OUTPUT );
  } else {
    #print "/sbin/ping -c 4 $server\n";
    open ( OUTPUT, "/sbin/ping -c 4 $server |" );
    while ( <OUTPUT> ) {
      $up = 1 if ( grep /bytes from/, $_ );
    }
    close ( OUTPUT );

    # old method 1
    #entman_logger( "log", "Using less reliable method to find server because hping is missing.  Server will be reported as up if the computer has network." );
    #my $network=`/usr/sbin/scutil -r $server`;
    #chomp $network;
    #$up = 1 if ( $network ne "Not Reachable" );

    # old method 2
    #  $network = `/sbin/ifconfig -a inet 2>/dev/null | 
    #  sed -n -e '/127.0.0.1/d' -e '/0.0.0.0/d' -e '/tunnel/d' -e '/inet/p' | wc -l`; 
    #  if ( $network == 0 ) {

  }
  if ( ! $up ) {
    entman_logger( "drawer log", "$server:$port is unreachable" );
  }
  return $up;
}

################################################################################
sub parse_radmind_line {
  my ( $line ) = @_;

  my %radmind_line = ();

  # check for comment
  if ( ( substr $line, 0, 1 ) eq "#" ) {
    $radmind_line{'comment'} = substr $line, 1, length ($line)-1;
    return %radmind_line;
  }
  
  my @theLine = split (' ', $line);

  # check for minus
  if ( $theLine[0] eq "-" ) {
    shift @theLine;
    $radmind_line{'minus'} = 1;
  }

  # check for plus (even needed?
  if ( $theLine[0] eq "+" ) {
    shift @theLine;
    $radmind_line{'plus'} = 1;
  }

  $radmind_line{'filetype'} = shift @theLine;

  $radmind_line{'path'} = shift @theLine;
  $radmind_line{'realpath'} = unescape_radmind_path ( $radmind_line{'path'} );
  if ( ( substr $radmind_line{'path'}, 0, 1 ) eq "." ) {
    $radmind_line{'relativepath'} = 1;
  } elsif ( ( substr $radmind_line{'path'}, 0, 1 ) eq "/" ) {
    $radmind_line{'absolutepath'} = 1;
  }

  if ($radmind_line{'filetype'} ne "l"  ) {

    $radmind_line{'perms'} = shift @theLine;
    $radmind_line{'owner'} = shift @theLine;
    $radmind_line{'group'} = shift @theLine;

    if ($radmind_line{'filetype'} ne "d"  ) {

      $radmind_line{'modtime'} = shift @theLine;
      $radmind_line{'size'} = shift @theLine;
      $radmind_line{'checksum'} = shift @theLine;

    } elsif ( $#theLine > 0 ) {
    
      $radmind_line{'appleinfo'} = shift @theLine;
    
    }

  } else {

    $radmind_line{'linkpath'} = shift @theLine;

  }

  return %radmind_line;

}

sub unescape_radmind_path {
  my ( $path ) = @_;
  $path =~ s/\\b/ /g;
#  $path =~ s/\\t/\t/g;; # convert tabs
#  $path =~ s/\\n/\n/g;; # convert newline
#  $path =~ s/\\r/\r/g;; # convert return
  $path =~ s/\\\\/\\/g; # convert backslashes
  return $path;
}

sub get_short_os_version {
  my $osnum_sht = get_os_version();
  $osnum_sht =~ s/(\d+\.\d+).*/$1/;
  return $osnum_sht;
}

sub get_os_version {
  my $osnum = `sw_vers | grep ProductVersion | awk '{print \$2}'`;
  chomp $osnum;
  return $osnum;
}

1;
