##
# File operations
##

# Version 8.3.6
# - Changed syntax of setPlistObjectForce so that date it requires -d or -a at 
# start (rather than assume a -d)
#
# Version 8.3.5
# - using File::Basename instead of calling shell's dirname command in saveDefaults
#
# Version 8.3.4
# - Changed syntax of setPlistObjectForce so that date is -t (instead of -d)
# and dict is -d (instead of -h)

use File::Basename;

sub expandFilePath {
  my ( $file ) = ( @_ );
  return NSString->stringWithFormat_( $file )->stringByExpandingTildeInPath->cString;
}

sub loadDefaults {
  my ( $file ) = ( @_ );
  if ( -e $file ) {
    return NSMutableDictionary->dictionaryWithContentsOfFile_( $file );
  } else {
    return;
  }
}

sub saveDefaults {
  my ( $dict, $file ) = ( @_ );
  if ( ( -f $file and -w $file ) or -w ( dirname "$file" ) ) {
    $dict->writeToFile_atomically_( $file, "0" );
    return 1;
  } else {
    return 0;
  }
}

##
# Read operations
##

sub getPlistObject {
  my ( $object, @keysIndexes ) = ( @_ );
  if ( @keysIndexes ) {
    foreach my $keyIndex ( @keysIndexes ) {
      if ( $object and $$object ) {
        if ( $object->isKindOfClass_( NSArray->class ) ) {
          my $count = $object->count;
          if ( $keyIndex < $count ) {
            $object = $object->objectAtIndex_( $keyIndex );
          } else {
            print STDERR "Tried to get an index that doesn't exist.  Count: $count\n";
            return;
          }
        } elsif ( $object->isKindOfClass_( NSDictionary->class ) ) {
          $object = $object->objectForKey_( $keyIndex );
        } else {
          print STDERR "Unknown type (not an array or a dictionary): $object\n";
          return;
        }
      } else {
        print STDERR "Got nil or other error for $keyIndex.\n";
        return;
      }
    }
  }
  return $object;
}

##
# Set operations
##

sub setPlistObjectForce {
  my ( $plistContainer, $pathRef ) = ( @_ );

  my $debug = 0;

  my $perlplist_dict_token = '-d';
  my $perlplist_array_token = '-a';
  my $perlplist_date_token = '-t';
  my $perlplist_int_token = '-i';
  my $perlplist_float_token = '-f';
  my $perlplist_bool_token = '-b';
  my $perlplist_string_token = '-s';

  my @path = @$pathRef;
  my $objectToSet = pop ( @path );

  my @keyesIndexesSoFar = ();

  my $last_container_type = shift ( @path );
  my $parentContainer = $plistContainer;

  for ( $ii = 0; $ii <= $#path; $ii+=2 ) {
    
    my $value_token = $path[$ii];
    my $data_type_token = $path[$ii+1];
    push @keyesIndexesSoFar, $value_token;

    print "keyesIndexesSoFar @keyesIndexesSoFar\n" if $debug;

    # find/create missing container
    my $do_I_exist = getPlistObject( $plistContainer, @keyesIndexesSoFar );
    my $make_change = 0;

    if ( ! $$do_I_exist ) {
      $make_change = 1;
      print "Missing: @keyesIndexesSoFar (current token: $data_type_token).\n";
    } else {
      if ( $data_type_token eq $perlplist_dict_token ) {
        if ( ! $do_I_exist->isKindOfClass_( NSDictionary->class ) ) {
          $make_change = 1;
          print "Should be dictionary but isn't: @keyesIndexesSoFar\n";
        } else {
          $parentContainer = $do_I_exist;
        }
      } elsif ( $data_type_token eq $perlplist_array_token ) {
        if ( ! $do_I_exist->isKindOfClass_( NSArray->class ) ) {
          $make_change = 1;
          print "Should be array but isn't: @keyesIndexesSoFar\n";
        } else {
          $parentContainer = $do_I_exist;
        }
      } else {
        $make_change = 1;
      }
    }

    if ( $make_change ) {

      my $addMe;
      if ( $data_type_token eq $perlplist_dict_token ) {
        $addMe = cocoaDictFromPerlHash( ( ) );
      } elsif ( $data_type_token eq $perlplist_array_token ) {
        $addMe = cocoaArrayFromPerlArray( ( ) );
      } elsif ( $data_type_token eq $perlplist_date_token ) {
        $addMe = cocoaDate( $objectToSet );
      } elsif ( $data_type_token eq $perlplist_int_token ) {
        $addMe = cocoaInt( $objectToSet );
      } elsif ( $data_type_token eq $perlplist_float_token ) {
        $addMe = cocoaFloat( $objectToSet );
      } elsif ( $data_type_token eq $perlplist_bool_token ) {
        $addMe = cocoaBool( $objectToSet );
      } elsif ( $data_type_token eq $perlplist_string_token ) {
        $addMe =  $objectToSet;
      }

      if ( $last_container_type eq $perlplist_array_token ) {
        if ( $parentContainer->isKindOfClass_( NSArray->class ) ) {

          while ( $value_token >= $parentContainer->count ) {
            $parentContainer->addObject_( "" );
          }

          $parentContainer->replaceObjectAtIndex_withObject_( $value_token, $addMe );

        } else {
          print STDERR "Tried to add array index to non-array container, stopping.\n";
          return;
        }
      } elsif ( $last_container_type eq $perlplist_dict_token ) {
        if ( $parentContainer->isKindOfClass_( NSDictionary->class ) ) {
          
          $parentContainer->setObject_forKey_( $addMe, $value_token );

        } else {
          print STDERR "Tried to add dictionary key-value pair to non-dictionary container, stopping.\n";
          return;
        }
      }
      $parentContainer = $addMe;
    }
    $last_container_type = $data_type_token;
  }
}

sub setPlistObject {
  my ( $plistContainer, @keyesIndexesValue ) = ( @_ );
  my $objectToSet = pop ( @keyesIndexesValue );
  my $keyIndex = pop ( @keyesIndexesValue );
  my $parentContainer = getPlistObject ( $plistContainer, @keyesIndexesValue );
  if ( $parentContainer and $$parentContainer ) {
    if ( $parentContainer->isKindOfClass_( NSArray->class ) ) {
      if ( $keyIndex > $parentContainer->count -1 ) {
        $parentContainer->addObject_( $objectToSet );
      } else {
        $parentContainer->replaceObjectAtIndex_withObject_( $keyIndex, $objectToSet );
      }
    } elsif ( $parentContainer->isKindOfClass_( NSDictionary->class ) ) {
      $parentContainer->setObject_forKey_( $objectToSet, $keyIndex );
    } else {
      print STDERR "Unknown parent container type.\n";
    }
  } else {
    print STDERR "Could not get value specified by @keyesIndexesValue.\n";
  }
}

sub removePlistObject {
  my ( $plistContainer, @keyesIndexesValue ) = ( @_ );
  my $keyIndex = pop ( @keyesIndexesValue );
  my $success = 0;
  my $parentContainer = getPlistObject ( $plistContainer, @keyesIndexesValue );
  if ( $parentContainer and $$parentContainer ) {
    if ( $parentContainer->isKindOfClass_( NSArray->class ) ) {
      if ( $keyIndex > $parentContainer->count -1 ) {
        print "removePlistObject: I don't think this should be possible\n";
        # don't think this should be possible
        #$parentContainer->addObject_( $objectToSet );
      } else {
        $parentContainer->removeObjectAtIndex_( $keyIndex );
        $success = 1;
      }
    } elsif ( $parentContainer->isKindOfClass_( NSDictionary->class ) ) {
      $parentContainer->removeObjectForKey_( $keyIndex );
      $success = 1;
    } else {
      print STDERR "Unknown parent container type.\n";
    }
  } else {
    print STDERR "Could not get value specified by @keyesIndexesValue.\n";
  }
  return $success;
}

##
# Convert operations
##

sub perlArrayFromNSArray {
  my ( $cocoaArray, $convertAll ) = ( @_ );
  my @perlArray = ();
  my $enumerator = $cocoaArray->objectEnumerator();
  my $value;
  while ( $value = $enumerator->nextObject() and $$value ) {
    if ( substr ( ref ( $value ), 0,9 ) eq "NSCFArray" ) {
      my @newarray = perlArrayFromNSArray( $value, $convertAll );
      push (@perlArray, \@newarray);
    } elsif ( substr ( ref ( $value ), 0,14 ) eq "NSCFDictionary" ) {
      my %newhash = perlHashFromNSDict( $value, $convertAll );
      push (@perlArray, \%newhash);
    } else {
      push ( @perlArray, $convertAll ? perlValue( $value ) : $value );
    }
  }
  return @perlArray;
}

sub perlHashFromNSDict {
  my ( $cocoaDict, $convertAll ) = ( @_ );
  my %perlHash = ();
  my $enumerator = $cocoaDict->keyEnumerator();
  my $key;
  while ( $key = $enumerator->nextObject() and $$key ) {
    my $value = $cocoaDict->objectForKey_($key);
    if ( $value ) { # check to make sure an object was set
      if ( substr ( ref ( $value ), 0,9 ) eq "NSCFArray" ) {
        my @newarray = perlArrayFromNSArray( $value, $convertAll );
        $perlHash{ perlValue( $key ) } = \@newarray;
      } elsif ( substr ( ref ( $value ), 0,14 ) eq "NSCFDictionary" ) {
        my %newHash = perlHashFromNSDict( $value, $convertAll );
        $perlHash{ perlValue( $key ) } = \%newHash;
      } else {
        $perlHash{ perlValue( $key ) } = 
        $convertAll ? perlValue( $value ) : $value;
      }
    }
  }
  return %perlHash;
}

sub cocoaDictFromPerlHash {
  my ( %hash ) = ( @_ );
  my $cocoaDict = NSMutableDictionary->dictionary();
  while ( my ( $key, $value ) = each( %hash ) ) {
    if ( defined $value ) {
      if ( ref ( $value ) eq "ARRAY" ) {
        $cocoaDict->setObject_forKey_( cocoaArrayFromPerlArray( @$value ), 
                                       $key );
      } elsif ( ref ( $value ) eq "HASH" ) {
        $cocoaDict->setObject_forKey_( cocoaDictFromPerlHash( %$value ), 
                                       $key );
      } elsif ( ref ( $value ) eq "SCALAR" ) {
        $cocoaDict->setObject_forKey_( $$value, $key );
      } elsif ( substr ( ref ( $value ), 0,4 ) eq "NSCF" ) {
        $cocoaDict->setObject_forKey_( $value, $key );
      } else {
        $cocoaDict->setObject_forKey_( "$value", $key );
      }
    } else {
      print STDERR "The value was not defined for $key!\n";
    }
  }
  return $cocoaDict;
}

sub cocoaArrayFromPerlArray {
  my ( @perlArray ) = ( @_ );
  my $cocoaArray = NSMutableArray->array();
  foreach my $value ( @perlArray ) {
    if ( defined $value ) {
      if ( ref ( $value ) eq "ARRAY" ) {
        $cocoaArray->addObject_( cocoaArrayFromPerlArray( @$value ) );
      } elsif ( ref ( $value ) eq "HASH" ) {
        $cocoaArray->addObject_( cocoaDictFromPerlHash( %$value ) );
      } elsif ( ref ( $value ) eq "SCALAR" ) {
        $cocoaArray->addObject_( $$value );
      } elsif ( substr ( ref ( $value ), 0,4 ) eq "NSCF" ) {
        $cocoaArray->addObject_( $value );
      } else {
        $cocoaArray->addObject_( "$value" );
      }
    } else {
      print STDERR "The value was not defined!\n";
    }
  }
  return $cocoaArray;
}

sub cocoaInt {
  return NSNumber->numberWithLong_( $_[0] );
}

sub cocoaBool {
  return NSNumber->numberWithBool_( $_[0] );
}

sub cocoaFloat {
  return NSNumber->numberWithDouble_( $_[0] );
}

sub cocoaDate {
  return NSDate->dateWithTimeIntervalSince1970_( $_[0] );
}

##
# Other options
##

sub printObject {
  my ( $value ) = ( @_ );
  if ( $$value and $value ) {
    print $value->description->cString() . "\n";
    return $value->description->cString();
  } else {
    print "defaults had no value\n";
  }
}

sub perlValue {
  my ( $object ) = @_;
  return $object->description()->UTF8String();
}

1;
