#!/usr/bin/perl -w

################################################################################
# Copyright (c) 2006 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

use lib "/Library/Xhooks/Modules";

use Foundation;
require "xhooks/lib/perlplist.pl";
use Getopt::Std;

usage() if ! defined $ARGV[0];


#####
# Get options
#
getopts( "dfr:s:", \%options );

my $router;
my $debug;
my $subnetmask;
my $force_change;

if ( defined $options{'r'} ) {
  $router = $options{'r'};
} else {
  $router = "";
}

if ( defined $options{'s'} ) {
  $subnetmask = $options{'s'};
} else {
  $subnetmask = "";
}

if ( defined $options{'f'} ) {
  $force_change = 1;
} else {
  $force_change = 0;
}

if ( defined $options{'d'} ) {
  $debug = 1;
} else {
  $debug = 0;
}

#####

my $networkPrefs = defined $ARGV[1] ? $ARGV[1] : "/Library/Preferences/SystemConfiguration/preferences.plist";
print "Modifying $networkPrefs\n";

#####
# Load prefs
#
my $dict;
if ( -e $networkPrefs ) {
  $dict = loadDefaults( $networkPrefs );
} else {
  die "The file: $networkPrefs not found";
}

my $currentSet = perlValue(getPlistObject($dict, "CurrentSet"));
$currentSet =~ s/\/Sets\///;
print "currentSet $currentSet\n" if $debug;

@serviceOrder = perlArrayFromNSArray(getPlistObject($dict, "Sets", $currentSet, "Network", "Global", "IPv4", "ServiceOrder"));

my $ip;

my $made_change = 0;

########
# Find "manual" config method and change its IP
#
foreach $ii ( @serviceOrder ) {
	my $serviceKey = perlValue($ii);
	print "serviceKey $serviceKey\n" if $debug;
	my $configMethod = perlValue(getPlistObject($dict, "NetworkServices", $serviceKey, "IPv4", "ConfigMethod"));
	print "configMethod $configMethod\n" if $debug;

	if ( $configMethod eq "Manual" ) {
		setPlistObject($dict, "NetworkServices", $serviceKey, "IPv4", "Addresses", 0, $ARGV[0]);

		if ( $router ne "" ) {
			setPlistObject($dict, "NetworkServices", $serviceKey, "IPv4", "Router", $router);
		}
		if ( $subnetmask ne "" ) {
			setPlistObject($dict, "NetworkServices", $serviceKey, "IPv4", "SubnetMasks", 0, $subnetmask);
		}

		$made_change = 1;

		last;
	}
}

#####
# If "manual" config method not found, just change the first service to "manual"
#
if ( $force_change and ! $made_change ) {
	print "Did not find Manual config method, changing the first service to Manual.\n";
	setPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "ConfigMethod", "Manual");

	# Add Addresses array if it doesn't exist
	my $result = getPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "Addresses" );
	if ( ! $$result ) {
		my @addresses = cocoaArrayFromPerlArray( ( $ARGV[0] ) );
		setPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "Addresses", @addresses);
	} else {
		setPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "Addresses", 0, $ARGV[0]);
	}
	
	if ( $router ne "" ) {
		setPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "Router", $router);
	}

	# Add subnet mask
	if ( $subnetmask ne "" ) {
		$result = getPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "SubnetMasks" );
		if ( ! $$result ) {
			my @subnetmasks = cocoaArrayFromPerlArray( ( $subnetmask ) );
			setPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "SubnetMasks", @subnetmasks);
		} else {
			setPlistObject($dict, "NetworkServices", $serviceOrder[0], "IPv4", "SubnetMasks", 0, $subnetmask);
		}
	}
}

#####
# Save file
#
saveDefaults( $dict, $networkPrefs );

sub usage {
	print "$0 12.34.56.78 [path to file]\n";
	print "If no path is given, it will use /Library/Preferences/SystemConfiguration/preferences.plist.\n";
	exit
  
}

exit 0;
