#!/usr/bin/perl

use Foundation;
use lib "/Library/Xhooks/Modules";
require "xhooks/lib/perlplist.pl";

usage() if ( $#ARGV < 2 );

my $filename = shift @ARGV;

my $dict;
if ( -e $filename ) {
  $dict = loadDefaults( $filename );
} else {
  print "The file $filename does not exist, creating a new file.\n";
  $dict = NSMutableDictionary->dictionary();
}

setPlistObjectForce( $dict, \@ARGV );

printObject ($dict);

saveDefaults( $dict, $filename );

###############################################################################
sub usage{
  print <<EOF;
Usage $0 <filename> <container flag> <key|index> [<container flag> <key|index>...] <datatype flag> <value>

	This script will modify a plist file according to the container path specified
	and the datatype specified.  It can only set one value at a time (but can
	create an entire path all at once if it is missing).

	Container flags:

		-d = dictionary
		-a = array
		
		If an array index is specified that does not exist, it will create indexes
		with blank values that are the same datatype as the last value.  Indexes
		are zero based.
		
		Specifying a key or index that already exists will overwrite any existing
		values without warning!

	Datatype flags:

		-s = string
		-i = integer
		-f = float
		-b = boolean
		-t = date

	Examples:

		$0 /path/to/file.plist -d ABC -s "Stringy"

		<dict>
			<key>ABC</key>
			<string>Stringy</string>
		</dict>

		$0 /path/to/file.plist -d List -a 3 -s "I'm a string"

		<dict>
			<key>List</key>
			<array>
				<string></string>
				<string></string>
				<string></string>
				<string>I'm a string</string>
			</array>
		</dict>

		$0 /path/to/file.plist -d Dates -d "A good day" -t 121766400

		<dict>
			<key>Dates</key>
			<dict>
				<key>A good day</key>
				<date>1973-11-10T08:00:00Z</date>
			</dict>
		</dict>

		$0 /path/to/file.plist -d true -b 0

		<dict>
			<key>true</key>
			<false/>
		</dict>

EOF

  exit;
}
